/*
 * shader.h
 *
 *  Created on: 02 . 2014 .
 *      Author: Allex
 */

#ifndef SHADER_H_
#define SHADER_H_

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include "utils.h"

class cDemoSceneShader{
protected:
	unsigned int		m_ShaderProgram;

	unsigned int 		m_PositionAttrib;
	unsigned int		m_NormalAttrib;

	unsigned int		m_ViewProjectionMatrixUniform;
	unsigned int		m_ModelMatrixUniform;
	unsigned int		m_ColorUniform;
	unsigned int		m_LightEnabledUniform;
public:
	unsigned int getPositionAttrib(){
		return m_PositionAttrib;
	}
	unsigned int getNormalAttrib(){
		return m_NormalAttrib;
	}

	void setProgram(int ShaderProgram){
		m_ShaderProgram = ShaderProgram;

		glUseProgram(m_ShaderProgram);

		m_PositionAttrib = glGetAttribLocation(m_ShaderProgram, "vPosition");
		m_NormalAttrib = glGetAttribLocation(m_ShaderProgram, "vNormal");

		m_ViewProjectionMatrixUniform = glGetUniformLocation(m_ShaderProgram,"ViewProjectionMatrix");
		m_ModelMatrixUniform = glGetUniformLocation(m_ShaderProgram,"ModelMatrix");
		m_ColorUniform = glGetUniformLocation(m_ShaderProgram,"Color");
		m_LightEnabledUniform = glGetUniformLocation(m_ShaderProgram,"LightEnabled");
	}

	void setObjectParams(const float* ModelMatrix, float R, float G, float B, float A, bool LightEnabled){
		glUniformMatrix4fv(m_ModelMatrixUniform, 1, GL_FALSE, ModelMatrix);
		glUniform4f(m_ColorUniform,R,G,B,A);
		glUniform1f(m_LightEnabledUniform,LightEnabled?1:0);
	}

	void bind(const float* ViewProjectionMatrix){
		glUseProgram(m_ShaderProgram);
		glUniformMatrix4fv(m_ViewProjectionMatrixUniform, 1, GL_FALSE, ViewProjectionMatrix);
	}

	void unbind(){
		glUseProgram(0);
	}

};


class cDemoFloorShader{
protected:
	unsigned int		m_ShaderProgram;

	unsigned int		m_TexCoordAttrib;

	unsigned int		m_ViewProjectionMatrixUniform;
public:
	unsigned int getTexCoordAttrib(){
		return m_TexCoordAttrib;
	}

	void setProgram(int ShaderProgram){
		m_ShaderProgram = ShaderProgram;

		glUseProgram(m_ShaderProgram);

		m_TexCoordAttrib = glGetAttribLocation(m_ShaderProgram, "vTexCoord");

		m_ViewProjectionMatrixUniform = glGetUniformLocation(m_ShaderProgram,"ViewProjectionMatrix");

		glUniform1i(glGetUniformLocation(m_ShaderProgram,"texture"),0);
	}

	void bind(const float* ViewProjectionMatrix){
		glUseProgram(m_ShaderProgram);
		glUniformMatrix4fv(m_ViewProjectionMatrixUniform, 1, GL_FALSE, ViewProjectionMatrix);
	}

	void unbind(){
		glUseProgram(0);
	}

};

#endif /* SHADER_H_ */
