package com.geometros.javademo;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;


import android.content.Context;
import android.opengl.GLES20;

public class Shader {
	protected static int loadShader(int type, String shaderCode){

	    // create a vertex shader type (GLES20.GL_VERTEX_SHADER)
	    // or a fragment shader type (GLES20.GL_FRAGMENT_SHADER)
	    int shader = GLES20.glCreateShader(type);

	    // add the source code to the shader and compile it
	    GLES20.glShaderSource(shader, shaderCode);
	    GLES20.glCompileShader(shader);

	    return shader;
	}
	
	protected static String resToSource(Context context, int Resource){
		StringBuffer buffer = new StringBuffer();

		try {
			InputStream inputStream = context.getResources().openRawResource(Resource);
			// setup Bufferedreader
			BufferedReader in = new BufferedReader(new InputStreamReader(inputStream));

			String read = in.readLine();
			while (read != null) {
				buffer.append(read + "\n");
				read = in.readLine();
			}

			buffer.deleteCharAt(buffer.length() - 1);

		} catch (Exception e) {
			android.util.Log.v("ERROR-readingShader", "Could not read shader: " + e.getLocalizedMessage());
		}

		return buffer.toString();
	}

	private int vertexShader;
    private int fragmentShader;
    private int program;
    
    protected void init(String vertexShaderSourceCode, String fragmentShaderSourceCode){
    	vertexShader = loadShader(GLES20.GL_VERTEX_SHADER, vertexShaderSourceCode);
    	fragmentShader = loadShader(GLES20.GL_FRAGMENT_SHADER, fragmentShaderSourceCode);
    	
    	program = GLES20.glCreateProgram();              
        GLES20.glAttachShader(program, vertexShader);   
        GLES20.glAttachShader(program, fragmentShader); 
        GLES20.glLinkProgram(program);
        GLES20.glUseProgram(program);
    }
    
	public Shader(String vertexShaderSourceCode, String fragmentShaderSourceCode){
		init(vertexShaderSourceCode, fragmentShaderSourceCode);
	}
	
	public Shader(Context context, int vertexResource, int fragmentResource){
		init(resToSource(context,vertexResource),resToSource(context,fragmentResource));
	}
	
	public int getUniform(String Name){
		return GLES20.glGetUniformLocation(program, Name);
	}
	
	public void setUniform1i(int Location, int Value){
		GLES20.glUniform1i(Location, Value);
	}
	
	public void setUniform1f(int Location, float Value){
		GLES20.glUniform1f(Location, Value);
	}
	
	public void setUniform2f(int Location, float v1, float v2){
		GLES20.glUniform2f(Location, v1,v2);
	}
	
	public void setUniform3f(int Location, float v1, float v2, float v3){
		GLES20.glUniform3f(Location, v1,v2,v3);
	}
	
	public void setUniform4f(int Location, float v1, float v2, float v3, float v4){
		GLES20.glUniform4f(Location, v1,v2,v3,v4);
	}
	
	public void setUniformMatrix4(int Location, float[] Matrix){
		GLES20.glUniformMatrix4fv(Location, 1, false, Matrix,0);
	}
		
	public int getAttribLocation(String Name){
		return GLES20.glGetAttribLocation(program, Name);
	}
	
	public void bind(){
		GLES20.glUseProgram(program);
	}
	
	public void unbind(){
		GLES20.glUseProgram(0);
	}
	
	public int getGLProgram(){
		return program;
	}
}
