
#import "ViewController.h"
#import <SpriteKit/SpriteKit.h>

#import "sgObjectNode.h"

#define COLOR_GRAY       0
#define COLOR_GREEN      1
#define COLOR_WHITE      2
#define COLOR_BLUE       3
#define COLOR_RED        4
#define COLOR_YELLOW     5
#define COLOR_GRAY_TR    6
#define COLOR_GREEN_TR   7
#define COLOR_WHITE_TR   8
#define COLOR_BLUE_TR    9
#define COLOR_RED_TR     10
#define COLOR_YELLOW_TR  11


static bool load_fonts()
{
    
    NSFileManager *fm = [NSFileManager defaultManager];
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    
    NSString* dirStrNew1 = [NSString stringWithFormat:@"%@/Helvetica.shx", documentsDirectory];
    NSString* path1 = [[NSBundle mainBundle] pathForResource:@"Helvetica" ofType:@"shx"];
    
    NSError *error = nil;
    
    [fm copyItemAtPath:path1 toPath:dirStrNew1 error:&error];
    
    
    {
        const char* cString = [dirStrNew1 cStringUsingEncoding:NSASCIIStringEncoding];
        
        
        sgCFont* fnt1 = sgCFont::LoadFont(cString,NULL,0);
        if (!fnt1)
        {
            return false;
        }
        if (!sgFontManager::AttachFont(fnt1))
        {
            sgCFont::UnloadFont(fnt1);
            return false;
        }
    }
    
    
    sgFontManager::SetCurrentFont(0);
    
    return true;
}


@implementation ViewController
{
    SCNNode*    _spotLightNode;
    SCNNode*    _cameraNode;
    SCNCamera*  camera;
    SCNNode*    mainObjectsParent;
}
            
- (void)viewDidLoad {
    [super viewDidLoad];
                                    
    sgInitKernel();
    sgC3DObject::AutoTriangulate(true, SG_DELAUNAY_TRIANGULATION, SG_SEPARATE_BUFFERS);
    load_fonts();
    
    //set the background to back
    [_scene_view setBackgroundColor:[NSColor whiteColor]];

    
    _scene_view.scene = [self setupScene];
    _scene_view.antialiasingMode = SCNAntialiasingModeMultisampling4X;
    
    _scene_view.pointOfView = _cameraNode;
    
    mainObjectsParent = [[SCNNode alloc] init];
    [_scene_view.scene.rootNode addChildNode:mainObjectsParent];
    
}

-(SCNScene*) setupScene
{
    // create a new scene
    SCNScene*  scene = [[SCNScene alloc] init];
    
    //global environment
    [self setupEnvironment:scene];
    
    //create a main camera
    camera.zNear = 200;
    camera.zFar = 2000;
    
    _cameraNode.camera = camera;
    
    _cameraNode.position = SCNVector3Make(0, 500, 500);
    _cameraNode.rotation  = SCNVector4Make(1, 0, 0, 5.5890);
    
    [scene.rootNode addChildNode:_cameraNode];
    
    return scene;
}

-(void) setupEnvironment:(SCNScene*)scene
{
    // add an ambient light
    SCNNode* ambientLight = [[SCNNode alloc] init];
    ambientLight.light = [[SCNLight alloc] init];
    ambientLight.light.type = SCNLightTypeAmbient;
    ambientLight.light.color = [NSColor  colorWithDeviceWhite:0.5 alpha:0.1];
    [scene.rootNode addChildNode:ambientLight];
    
    SCNNode* lightNode = [[SCNNode alloc] init];
    lightNode.light = [[SCNLight alloc] init];
    lightNode.light.type = SCNLightTypeSpot;
    lightNode.light.castsShadow = false;
    lightNode.light.color = [NSColor  colorWithDeviceWhite:0.8 alpha:1.0];
    lightNode.position = SCNVector3Make(0, 1000, 1200);
    lightNode.rotation = SCNVector4Make(1, 0, 0, -1.122);
    lightNode.light.spotInnerAngle = 0;
    lightNode.light.spotOuterAngle = 100;
    lightNode.light.shadowColor = [SKColor blackColor] ;
    lightNode.light.zFar = 200;
    lightNode.light.zNear = 100;
    [scene.rootNode addChildNode:lightNode];
    
    _spotLightNode = lightNode;
    
    //floor
    SCNFloor* floorGeo = [[SCNFloor alloc] init];
    floorGeo.firstMaterial.diffuse.contents = @"floor.png";
    floorGeo.firstMaterial.diffuse.contentsTransform = SCNMatrix4MakeScale(1.5, 1.5, 1);
    floorGeo.firstMaterial.locksAmbientWithDiffuse = true;
    
    SCNNode* floor = [[SCNNode alloc] init];
    floor.geometry = floorGeo;
    
    
    
    [scene.rootNode addChildNode:floor];
}


- (void)setRepresentedObject:(id)representedObject {
    [super setRepresentedObject:representedObject];
    
    // Update the view, if already loaded.
    
}

-(void) clearScene
{
    [mainObjectsParent enumerateChildNodesUsingBlock:^(SCNNode *child, BOOL *stop) {
        if ([child isKindOfClass:[sgObjectNode class]]) {
            [child removeFromParentNode];
        }
    }];
    sgGetScene()->Clear();
}

-(void) addOneObject:(sgCObject*)obj
{
    if (obj->GetType()!=SG_OT_GROUP && obj->GetType()!=SG_OT_CONTOUR)
    {
        sgObjectNode* newNode = [[sgObjectNode alloc] initWithObject:&obj];
        [mainObjectsParent addChildNode:newNode];
    }
    else
    {
        if (obj->GetType()==SG_OT_GROUP)
        {
            sgCGroup* grObj = (sgCGroup*)obj;
            
            sgCObject* curObj = grObj->GetChildrenList()->GetHead();
            
            while (curObj)
            {
                [self addOneObject:curObj];
                curObj = grObj->GetChildrenList()->GetNext(curObj);
            }
        }
        else
            if (obj->GetType()==SG_OT_CONTOUR)
            {
                sgCContour* contObj = (sgCContour*)obj;
                
                sgCObject* curObj = contObj->GetChildrenList()->GetHead();
                
                while (curObj)
                {
                    [self addOneObject:curObj];
                    curObj = contObj->GetChildrenList()->GetNext(curObj);
                }
            }
    }
}

-(void) fillScene
{
    sgCObject* curObj = sgGetScene()->GetObjectsList()->GetHead();
    
    while (curObj)
    {
        [self addOneObject:curObj];
        curObj = sgGetScene()->GetObjectsList()->GetNext(curObj);
    }
}

- (IBAction)PressScene:(id)sender
{
    long sceneIndex = ((NSButton*)sender).tag;
    [self clearScene];
    [self test:sceneIndex];
    [self fillScene];
}











-(sgC3DObject*)  get3DAfterBooleanAndDeleteGroup:(sgCGroup*)booleanResult
{
    if (!booleanResult)
        return NULL;
    int ChCnt = booleanResult->GetChildrenList()->GetCount();
    
    if (ChCnt>1)
    {
        // not one object after boolean operation
        return NULL;
    }
    
    sgCObject** allChilds = (sgCObject**)malloc(ChCnt*sizeof(sgCObject*));
    booleanResult->BreakGroup(allChilds);
    sgCObject::DeleteObject(booleanResult);
    sgCObject* firstObj = allChilds[0];
    
    free(allChilds);
    
    if (firstObj->GetType()!=SG_OT_3D)
    {
        sgDeleteObject(firstObj);
        return NULL;
    }
    
    return (sgC3DObject*)firstObj;
}

-(void)      addPage:(sgFloat)pageWidth shiftX:(sgFloat)shX shiftY:(sgFloat)shY withContent:(bool)cont
{
    sgFloat pageHeight = pageWidth/sqrt(2.0f);   // A4 page proportions
    
    SG_POINT corn1 = {-pageWidth/2.0f + shX, -pageHeight/2.0f + shY, 0.0f};
    SG_POINT corn2 = {pageWidth/2.0f + shX, -pageHeight/2.0f + shY, 0.0f};
    SG_POINT corn3 = {pageWidth/2.0f + shX, pageHeight/2.0f + shY, 0.0f};
    SG_POINT corn4 = {-pageWidth/2.0f + shX, pageHeight/2.0f + shY, 0.0f};
    
    sgGetScene()->AttachObject(sgCreateLine(corn1.x, corn1.y, corn1.z, corn2.x, corn2.y, corn2.z));
    sgGetScene()->AttachObject(sgCreateLine(corn2.x, corn2.y, corn2.z, corn3.x, corn3.y, corn3.z));
    sgGetScene()->AttachObject(sgCreateLine(corn3.x, corn3.y, corn3.z, corn4.x, corn4.y, corn4.z));
    sgGetScene()->AttachObject(sgCreateLine(corn4.x, corn4.y, corn4.z, corn1.x, corn1.y, corn1.z));
    
    if (cont)
    {
        sgGetScene()->AttachObject(sgCreateLine(corn2.x - pageWidth/2, corn2.y, corn2.z, corn2.x - pageWidth/2, corn2.y+pageHeight/4, corn2.z));
        sgGetScene()->AttachObject(sgCreateLine(corn2.x - pageWidth/2, corn2.y+pageHeight/4, corn2.z, corn2.x, corn2.y+pageHeight/4, corn2.z));
        
        sgGetScene()->AttachObject(sgCreateLine(corn2.x - pageWidth/4, corn2.y, corn2.z, corn2.x - pageWidth/4, corn2.y+pageHeight/8, corn2.z));
        sgGetScene()->AttachObject(sgCreateLine(corn2.x - pageWidth/2, corn2.y+pageHeight/8, corn2.z, corn2.x, corn2.y+pageHeight/8, corn2.z));
        
        sgGetScene()->AttachObject(sgCreateLine(corn2.x - pageWidth/4, corn2.y+pageHeight/16, corn2.z, corn2.x, corn2.y+pageHeight/16, corn2.z));
        //sgGetScene()->AttachObject(sgCreateLine(corn2.x - pageWidth/4, corn2.y+3*pageHeight/16, corn2.z, corn2.x, corn2.y+3*pageHeight/16, corn2.z));
        
        
        [self addText:@"Geometros" Size:15 OnX:corn2.x - pageWidth/4.2 OnY:corn2.y+pageHeight/16+2];
        [self addText:@"sgCore" Size:15 OnX:corn2.x - pageWidth/5.2 OnY:corn2.y+2];
        
        
    }
    
}

-(void)    addRoundRect:(sgFloat)rectW withHeight:(sgFloat)rectH radius:(sgFloat)cornerRadius shiftX:(sgFloat)shX shiftY:(sgFloat)shY
{
    sgCObject*  contObjs[8];
    
    SG_POINT     minPnt = {shX, shY ,0};
    SG_POINT     maxPnt = {shX+rectW, shY+rectH ,0};
    
    SG_POINT    p1 = {minPnt.x+cornerRadius, maxPnt.y, 0};
    SG_POINT    p2 = {maxPnt.x-cornerRadius, maxPnt.y, 0};
    SG_POINT    p3 = {maxPnt.x, maxPnt.y-cornerRadius,  0};
    SG_POINT    p4 = {maxPnt.x, minPnt.y+cornerRadius,  0};
    SG_POINT    p5 = {maxPnt.x-cornerRadius, minPnt.y, 0};
    SG_POINT    p6 = {minPnt.x+cornerRadius, minPnt.y, 0};
    SG_POINT    p7 = {minPnt.x, minPnt.y+cornerRadius,  0};
    SG_POINT    p8 = {minPnt.x, maxPnt.y-cornerRadius, 0};
    
    SG_POINT    c1 = {p2.x, p3.y, 0};
    SG_POINT    c2 = {p5.x, p4.y, 0};
    SG_POINT    c3 = {p6.x, p7.y, 0};
    SG_POINT    c4 = {p1.x, p8.y, 0};
    
    
    contObjs[0] = sgCreateLine(p1.x, p1.y, p1.z, p2.x, p2.y, p2.z);
    SG_ARC  arc1;   arc1.FromCenterBeginEnd(c1, p2, p3, false);
    contObjs[1] = sgCreateArc(arc1);
    
    contObjs[2] = sgCreateLine(p3.x, p3.y, p3.z, p4.x, p4.y, p4.z);
    SG_ARC  arc2;   arc2.FromCenterBeginEnd(c2, p4, p5, false);
    contObjs[3] = sgCreateArc(arc2);
    
    contObjs[4] = sgCreateLine(p5.x, p5.y, p5.z, p6.x, p6.y, p6.z);
    SG_ARC  arc3;   arc3.FromCenterBeginEnd(c3, p6, p7, false);
    contObjs[5] = sgCreateArc(arc3);
    
    contObjs[6] = sgCreateLine(p7.x, p7.y, p7.z, p8.x, p8.y, p8.z);
    SG_ARC  arc4;   arc4.FromCenterBeginEnd(c4, p8, p1, false);
    contObjs[7] = sgCreateArc(arc4);
    
    sgCContour* contour = sgCContour::CreateContour(&contObjs[0], 8);
    
    if (!contour)
    {
        for (int i=0;i<8;i++)
            if (contObjs[i])
                sgDeleteObject(contObjs[i]);
        
        return;
    }
    
    sgGetScene()->AttachObject(contour);
    sgGetScene()->AttachObject(contour->GetEquidistantContour(cornerRadius/2, cornerRadius/2, false));
    
}

-(void)      addText:(NSString*)str Size:(sgFloat)txtSz OnX:(sgFloat)posX OnY:(sgFloat)posY
{
    SG_TEXT_STYLE    text_style;
    
    text_style.state = 0;
    text_style.height= txtSz;
    text_style.proportions= 100.0;
    text_style.angle= 0.0;
    text_style.horiz_space_proportion= 0.0;
    text_style.vert_space_proportion= 50.0;
    
    sgCText*  textO = sgCText::Create(sgFontManager::GetFont(sgFontManager::GetCurrentFont()),	text_style,
                                      [str cStringUsingEncoding:NSASCIIStringEncoding]);
    if (textO)
    {
        SG_VECTOR  textOrigin = {posX, posY, 0};
        textO->InitTempMatrix()->Translate(textOrigin);
        textO->ApplyTempMatrix();
        textO->DestroyTempMatrix();
        sgGetScene()->AttachObject(textO);
    }
    
    
}

-(void)  addDimension:(SG_DIMENSION_TYPE)dimType fromPoint:(SG_POINT)pntA toPoint:(SG_POINT)pntB midPoint:(SG_POINT)midP //withLabel:(NSString*)string
{
    SG_DIMENSION_STYLE        dim_style;
    
    dim_style.behaviour_type = SG_DBT_PARALLEL;
    dim_style.dimension_line = true;
    dim_style.first_side_line = true;
    dim_style.second_side_line = true;
    dim_style.lug_size = 0.1;
    
    dim_style.automatic_arrows = true;
    
    dim_style.out_first_arrow = false;
    dim_style.first_arrow_style = 1;
    
    dim_style.out_second_arrow= false;
    dim_style.second_arrow_style= 1;
    
    dim_style.arrows_size= 25;
    
    dim_style.text_align = SG_TA_CENTER;
    
    dim_style.text_style.state = 0;
    dim_style.text_style.height= 10;
    dim_style.text_style.proportions= 100.0;
    dim_style.text_style.angle= 0.0;
    dim_style.text_style.horiz_space_proportion= 0.0;
    dim_style.text_style.vert_space_proportion= 50.0;
    
    dim_style.precision = 1;
    
    SG_POINT ppp[3];
    ppp[0] = pntA;
    ppp[1] = pntB;
    ppp[2] = midP;
    /* if (fabs(pntA.x-pntB.x)>fabs(pntA.y-pntB.y))
     {
     ppp[0].x = (float)pntB.x; ppp[0].y = (float)pntA.y;    ppp[0].z = 0;
     ppp[1].x = (float)pntA.x; ppp[1].y = (float)pntB.y;    ppp[1].z = 0;
     }
     else
     {
     ppp[0].x = (float)pntA.x; ppp[0].y = (float)pntB.y;    ppp[0].z = 0;
     ppp[1].x = (float)pntB.x; ppp[1].y = (float)pntA.y;    ppp[1].z = 0;
     }
     ppp[2].x = (float)midP.x; ppp[2].y = (float)midP.y;    ppp[2].z = 0;
     */
    sgCDimensions* dimO = sgCDimensions::Create(dimType, ppp,	sgFontManager::GetFont(0),dim_style, NULL/*[string cStringUsingEncoding:NSASCIIStringEncoding]*/);
    sgGetScene()->AttachObject(dimO);
}

-(void)      add3D:(sgC3DObject*)obj WithColor:(int)color OnPosX:(sgFloat)posX OnPosY:(sgFloat)posY OnPosZ:(sgFloat)posZ
{
    if (!obj)
        return;
    
    SG_MATERIAL mm;
    obj->SetMaterial(mm); // need for UV calculation
    
    SG_POINT  shift = {posX, posY, posZ};
    obj->InitTempMatrix()->Translate(shift);
    obj->ApplyTempMatrix();
    obj->DestroyTempMatrix();
    
    obj->SetAttribute(SG_OA_COLOR, color);
    
    sgGetScene()->AttachObject(obj);
}


-(void)      test:(long)tInd
{
    SG_VECTOR zAxe = {0,0,1};
    SG_VECTOR yAxe = {0,1,0};
    SG_VECTOR xAxe = {1,0,0};
    
    SG_VECTOR zeroP = {0,0,0};
    
    const sgFloat pageW = 500.0f;
    switch (tInd) {
        case 0:
        {
            // Lines Page
            {
                [self addPage:pageW shiftX:0 shiftY:0 withContent:YES];
                //[sgCore addPage:pageW shiftX:0 shiftY:pageW-100 withContent:YES];
                //[sgCore addPage:pageW shiftX:0 shiftY:-(pageW-100) withContent:YES];
                
                [self addText:@"Lines" Size:30 OnX:15 OnY:-(pageW/2-120)];
                
                SG_POINT   Cen = {-pageW/4, pageW/10, 0};
                int fl=0;
                SG_POINT tmpPnt1, tmpPnt2;
                for (double i=0.0;i<2.0*M_PI;i+=M_PI/12.0)
                {
                    tmpPnt1.x = ((double)(fl%3+2)*pageW/20.0f)*cos(i)+Cen.x;
                    tmpPnt1.y = ((double)(fl%3+2)*pageW/20.0f)*sin(i)+Cen.y;
                    tmpPnt1.z = 0.0;
                    fl++;
                    tmpPnt2.x = ((double)(fl%3+2)*pageW/20.0f)*cos(i+M_PI/12.0)+Cen.x;
                    tmpPnt2.y = ((double)(fl%3+2)*pageW/20.0f)*sin(i+M_PI/12.0)+Cen.y;
                    tmpPnt2.z = 0.0;
                    
                    sgCLine* ln = sgCreateLine(tmpPnt1.x, tmpPnt1.y, tmpPnt1.z, tmpPnt2.x,tmpPnt2.y,tmpPnt2.z);
                    sgGetScene()->AttachObject(ln);
                }
                
                fl=0;
                for (double i=0.0;i<2.0*M_PI;i+=M_PI/12.0)
                {
                    tmpPnt1.x = ((double)(fl%3+2)*pageW/40.0f)*cos(i)+Cen.x;
                    tmpPnt1.y = ((double)(fl%3+2)*pageW/40.0f)*sin(i)+Cen.y;
                    tmpPnt1.z = 0.0;
                    fl++;
                    tmpPnt2.x = ((double)(fl%3+2)*pageW/40.0f)*cos(i+M_PI/12.0)+Cen.x;
                    tmpPnt2.y = ((double)(fl%3+2)*pageW/40.0f)*sin(i+M_PI/12.0)+Cen.y;
                    tmpPnt2.z = 0.0;
                    
                    sgCLine* ln = sgCreateLine(tmpPnt1.x, tmpPnt1.y, tmpPnt1.z, tmpPnt2.x,tmpPnt2.y,tmpPnt2.z);
                    sgGetScene()->AttachObject(ln);
                }
                
                SG_POINT p1 = {pageW/2.0+40, -pageW/sqrt(2.0f)/2.0 ,0};
                SG_POINT p2 = {pageW/2.0+40, pageW/sqrt(2.0f)/2.0, 0};
                SG_POINT p3 = {pageW/2.0+90, 0, 0};
                [self addDimension:SG_DT_LINEAR fromPoint:p2 toPoint:p1 midPoint:p3];
                
                SG_POINT p1_ = {-pageW/2.0, -pageW/sqrt(2.0f)/2.0-30 ,0};
                SG_POINT p2_ = {pageW/2.0, -pageW/sqrt(2.0f)/2.0-30, 0};
                SG_POINT p3_ = {0, -pageW/sqrt(2.0f)/2.0-30, 0};
                [self addDimension:SG_DT_LINEAR fromPoint:p1_ toPoint:p2_ midPoint:p3_];
                
                //Cen.x += pageW/5;
                SG_POINT pD_1 = {Cen.x , Cen.y + pageW/5, 0};
                SG_POINT pD_2 = {Cen.x , Cen.y - pageW/5 - 50, 0};
                [self addDimension:SG_DT_DIAM fromPoint:Cen toPoint:pD_1 midPoint:pD_2];
                
                //Cen.x -= pageW/5;
                //Cen.y -= pageW/5;
                SG_POINT pD2_1 = {Cen.x, Cen.y - pageW/10, 0};
                SG_POINT pD2_2 = {Cen.x + pageW/10 + 120, Cen.y , 0};
                [self addDimension:SG_DT_DIAM fromPoint:Cen toPoint:pD2_1 midPoint:pD2_2];
            }
            
            //Circles Page
            {
                [self addPage:pageW shiftX:0 shiftY:pageW-100 withContent:YES];
                
                SG_POINT   Cen = {-pageW/4, pageW/10+(pageW-100), 0};
                
                {
                    SG_CIRCLE  tmpCir;
                    tmpCir.FromCenterRadiusNormal(Cen, pageW/5, zAxe);
                    sgGetScene()->AttachObject(sgCreateCircle(tmpCir));
                }
                
                {
                    SG_CIRCLE  tmpCir;
                    tmpCir.FromCenterRadiusNormal(Cen, pageW/8, zAxe);
                    sgGetScene()->AttachObject(sgCreateCircle(tmpCir));
                }
                
                for (double i=0.0;i<2.0*M_PI;i+=M_PI/6.0f)
                {
                    SG_POINT   crCen = {static_cast<sgFloat>(pageW/6.2*cos(i)) + Cen.x,static_cast<sgFloat>(pageW/6.2*sin(i)) + Cen.y, 0.0};
                    SG_CIRCLE  crGeo;
                    crGeo.FromCenterRadiusNormal(crCen, pageW/35, zAxe);
                    sgGetScene()->AttachObject(sgCreateCircle(crGeo));
                }
                
                [self addText:@"Circles" Size:30 OnX:15 OnY:-(pageW/2-120)+(pageW-100)];
                
                SG_POINT p1 = {pageW/2.0+40, -pageW/sqrt(2.0f)/2.0 +(pageW-100),0};
                SG_POINT p2 = {pageW/2.0+40, pageW/sqrt(2.0f)/2.0 +(pageW-100), 0};
                SG_POINT p3 = {pageW/2.0+90, 0+(pageW-100), 0};
                [self addDimension:SG_DT_LINEAR fromPoint:p2 toPoint:p1 midPoint:p3];
                
                SG_POINT p1_ = {-pageW/2.0, -pageW/sqrt(2.0f)/2.0-30+(pageW-100) ,0};
                SG_POINT p2_ = {pageW/2.0, -pageW/sqrt(2.0f)/2.0-30+(pageW-100), 0};
                SG_POINT p3_ = {0, -pageW/sqrt(2.0f)/2.0-30+(pageW-100), 0};
                [self addDimension:SG_DT_LINEAR fromPoint:p1_ toPoint:p2_ midPoint:p3_];
                
                SG_POINT pD_1 = {Cen.x, Cen.y + pageW/5, 0};
                SG_POINT pD_2 = {Cen.x, Cen.y - pageW/5 - 50, 0};
                [self addDimension:SG_DT_DIAM fromPoint:Cen toPoint:pD_1 midPoint:pD_2];
                
                SG_POINT pD2_1 = {Cen.x, Cen.y - pageW/8, 0};
                SG_POINT pD2_2 = {Cen.x + pageW/8 + 120, Cen.y , 0};
                [self addDimension:SG_DT_DIAM fromPoint:Cen toPoint:pD2_1 midPoint:pD2_2];
                
                SG_POINT pD3_0 = {Cen.x+pageW/6.2, Cen.y, 0};
                SG_POINT pD3_1 = {pD3_0.x, pD3_0.y - pageW/35, 0};
                SG_POINT pD3_2 = {pD3_0.x + 50, pD3_0.y , 0};
                [self addDimension:SG_DT_DIAM fromPoint:pD3_0 toPoint:pD3_1 midPoint:pD3_2];
            }
        }
            break;
            //case 1:
        case 2:
        case 3:
        {
            [self addPage:pageW shiftX:0 shiftY:0 withContent:YES];
            
            SG_POINT   Cen = {-pageW/4, pageW/10, 0};
            sgFloat rad1 = pageW/5;
            sgFloat rad2 = pageW/10;
            
            SG_POINT  p1 = {static_cast<sgFloat>(rad1*cos(M_PI_4)) + Cen.x,static_cast<sgFloat>(rad1*sin(M_PI_4)) + Cen.y, 0.0};
            SG_POINT  p2 = {static_cast<sgFloat>(rad1*cos(-M_PI_4)) + Cen.x,static_cast<sgFloat>(rad1*sin(-M_PI_4)) + Cen.y, 0.0};
            SG_POINT  p3 = {static_cast<sgFloat>(rad2*cos(M_PI_4)) + Cen.x,static_cast<sgFloat>(rad2*sin(M_PI_4)) + Cen.y, 0.0};
            SG_POINT  p4 = {static_cast<sgFloat>(rad2*cos(-M_PI_4)) + Cen.x,static_cast<sgFloat>(rad2*sin(-M_PI_4)) + Cen.y, 0.0};
            
            
            {
                SG_ARC  tmpArc;
                tmpArc.FromCenterBeginEnd(Cen, p1, p2, true);
                sgGetScene()->AttachObject(sgCreateArc(tmpArc));
            }
            
            {
                SG_ARC  tmpArc;
                tmpArc.FromCenterBeginEnd(Cen, p3, p4, true);
                sgGetScene()->AttachObject(sgCreateArc(tmpArc));
            }
            
            sgGetScene()->AttachObject(sgCreateLine(p1.x, p1.y, p1.z, p3.x, p3.y, p3.z));
            sgGetScene()->AttachObject(sgCreateLine(p2.x, p2.y, p2.z, p4.x, p4.y, p4.z));
            
            [self addRoundRect:pageW/3 withHeight:pageW/3 radius:pageW/17 shiftX:pageW/20 shiftY:-pageW/20];
            
            if (tInd==2)
            {
                [self addText:@"Arcs" Size:30 OnX:15 OnY:-(pageW/2-120)];
            }
            else
            {
                sgCObject*  inCont = sgGetScene()->GetObjectsList()->GetTail();
                sgCObject*  outCont = sgGetScene()->GetObjectsList()->GetPrev(inCont);
                
                if (inCont && outCont && inCont->GetType()==SG_OT_CONTOUR && outCont->GetType()==SG_OT_CONTOUR)
                {
                    sgCContour* holeControur = (sgCContour*)inCont;
                    sgCContour* mainControur = (sgCContour*)outCont;
                    SG_VECTOR  extudeVector = {30,40, 100};
                    sgC3DObject* extrudeObj = (sgC3DObject*)sgKinematic::Extrude(*mainControur, (const sgC2DObject**)(&holeControur), 1, extudeVector, true);
                    [self add3D:extrudeObj WithColor:COLOR_BLUE OnPosX:0 OnPosY:0 OnPosZ:100];
                }
                
                SG_SPLINE* spl2 = SG_SPLINE::Create();
                int fl=0;
                SG_POINT tmpPnt;
                for (double i=0.0;i<2.0*3.14159265;i+=0.4)
                {
                    tmpPnt.x = ((double)(fl%3+2)*pageW/80.0f)*cos(i);
                    tmpPnt.y = ((double)(fl%3+2)*pageW/80.0f)*sin(i);
                    tmpPnt.z = 0.0;
                    spl2->AddKnot(tmpPnt,fl);
                    fl++;
                }
                spl2->Close();
                
                sgCSpline* spl2_obj = sgCreateSpline(*spl2);
                sgGetScene()->AttachObject(spl2_obj);
                
                SG_SPLINE::Delete(spl2);
                
                SG_POINT p1 = {-3.0f*pageW/20.0f,10,0};
                SG_POINT p2 = {-3.0f*pageW/20.0f,15,0};
                
                sgC3DObject* rO = (sgC3DObject*)sgKinematic::Rotation(*spl2_obj,p1,p2,270,true);
                
                SG_POINT trP = {3.0f*pageW/20.0f, 0, 0};
                rO->InitTempMatrix()->Translate(trP);
                rO->GetTempMatrix()->Rotate(zeroP, yAxe, M_PI/4.0f);
                rO->GetTempMatrix()->Rotate(zeroP, xAxe, M_PI/2.0f);
                
                rO->ApplyTempMatrix();
                rO->DestroyTempMatrix();
                
                
                spl2_obj->InitTempMatrix()->Translate(Cen);
                spl2_obj->ApplyTempMatrix();
                spl2_obj->DestroyTempMatrix();
                
                [self add3D:rO WithColor:COLOR_RED OnPosX:Cen.x OnPosY:Cen.y OnPosZ:140];
                
                
                [self addText:@"Extrude and Revolve" Size:15 OnX:15 OnY:-(pageW/2-130)];
                
            }
            
            
        }
            break;
        case 4:
        case 5:
        {
            //if (tInd==4)
            //  [sgCore addPage:pageW shiftX:0 shiftY:0 withContent:YES];
            
            SG_POINT tmpPnt;
            
            SG_SPLINE* spl1 = SG_SPLINE::Create();
            tmpPnt.x = -150.0; tmpPnt.y = 0.0; tmpPnt.z = 25.0;
            spl1->AddKnot(tmpPnt,0);
            tmpPnt.x = -125.0; tmpPnt.y = 0.0; tmpPnt.z = 100.0;
            spl1->AddKnot(tmpPnt,1);
            tmpPnt.x = -75.0; tmpPnt.y = 0.0; tmpPnt.z = 0.0;
            spl1->AddKnot(tmpPnt,2);
            tmpPnt.x = -50.0; tmpPnt.y = 0.0; tmpPnt.z = 0.0;
            spl1->AddKnot(tmpPnt,3);
            tmpPnt.x = -25.0; tmpPnt.y = 0.0; tmpPnt.z = 75.0;
            spl1->AddKnot(tmpPnt,4);
            tmpPnt.x =50.0; tmpPnt.y = 0.0; tmpPnt.z = 75.0;
            spl1->AddKnot(tmpPnt,5);
            tmpPnt.x =50.0; tmpPnt.y = 0.0; tmpPnt.z = 50.0;
            spl1->AddKnot(tmpPnt,6);
            tmpPnt.x =100.0; tmpPnt.y = 0.0; tmpPnt.z = 75.0;
            spl1->AddKnot(tmpPnt,7);
            
            sgCSpline* spl1_obj = sgCreateSpline(*spl1);
            SG_SPLINE::Delete(spl1);
            sgGetScene()->AttachObject(spl1_obj);
            
            
            
            SG_SPLINE* spl2 = SG_SPLINE::Create();
            
            tmpPnt.x = -125.0; tmpPnt.y = 100.0; tmpPnt.z = 100.0;
            spl2->AddKnot(tmpPnt,0);
            tmpPnt.x = -75.0; tmpPnt.y = 100.0; tmpPnt.z = 75.0;
            spl2->AddKnot(tmpPnt,1);
            tmpPnt.x = -50.0; tmpPnt.y = 100.0; tmpPnt.z = 100.0;
            spl2->AddKnot(tmpPnt,2);
            tmpPnt.x = -25.0; tmpPnt.y = 100.0; tmpPnt.z = 25.0;
            spl2->AddKnot(tmpPnt,3);
            tmpPnt.x =100.0; tmpPnt.y = 100.0; tmpPnt.z = 50.0;
            spl2->AddKnot(tmpPnt,4);
            
            sgCSpline* spl2_obj = sgCreateSpline(*spl2);
            SG_SPLINE::Delete(spl2);
            sgGetScene()->AttachObject(spl2_obj);
            
            
            SG_SPLINE* spl3 = SG_SPLINE::Create();
            
            tmpPnt.x = -150.0; tmpPnt.y = 0.0; tmpPnt.z = 25.0;
            spl3->AddKnot(tmpPnt,0);
            tmpPnt.x = -175.0; tmpPnt.y = 12.5; tmpPnt.z = 125.0;
            spl3->AddKnot(tmpPnt,1);
            tmpPnt.x = -100.0; tmpPnt.y = 50.0; tmpPnt.z = 25.0;
            spl3->AddKnot(tmpPnt,2);
            tmpPnt.x = -125.0; tmpPnt.y = 75.0; tmpPnt.z = 75.0;
            spl3->AddKnot(tmpPnt,3);
            tmpPnt.x =-125.0; tmpPnt.y = 100.0; tmpPnt.z = 100.0;
            spl3->AddKnot(tmpPnt,4);
            
            sgCSpline* spl3_obj = sgCreateSpline(*spl3);
            SG_SPLINE::Delete(spl3);
            sgGetScene()->AttachObject(spl3_obj);
            
            SG_SPLINE* spl4 = SG_SPLINE::Create();
            
            tmpPnt.x = 100.0; tmpPnt.y = 0.0; tmpPnt.z = 75.0;
            spl4->AddKnot(tmpPnt,0);
            tmpPnt.x = 75.0; tmpPnt.y = 25.0; tmpPnt.z = 100.0;
            spl4->AddKnot(tmpPnt,1);
            tmpPnt.x = 125.0; tmpPnt.y = 50.0; tmpPnt.z = 25.0;
            spl4->AddKnot(tmpPnt,2);
            tmpPnt.x = 75.0; tmpPnt.y = 75.0; tmpPnt.z = 62.5;
            spl4->AddKnot(tmpPnt,3);
            tmpPnt.x =100.0; tmpPnt.y = 100.0; tmpPnt.z = 50.0;
            spl4->AddKnot(tmpPnt,4);
            
            sgCSpline* spl4_obj = sgCreateSpline(*spl4);
            SG_SPLINE::Delete(spl4);
            sgGetScene()->AttachObject(spl4_obj);
            
            
            SG_SPLINE* spl5 = SG_SPLINE::Create();
            
            tmpPnt.x = -500.0; tmpPnt.y = 0.0; tmpPnt.z = 0.0;
            spl5->AddKnot(tmpPnt,0);
            tmpPnt.x = -700.0; tmpPnt.y = 300.0; tmpPnt.z = 0.0;
            spl5->AddKnot(tmpPnt,1);
            tmpPnt.x = -500.0; tmpPnt.y = 200.0; tmpPnt.z = 0.0;
            spl5->AddKnot(tmpPnt,2);
            tmpPnt.x = -400.0; tmpPnt.y = 400.0; tmpPnt.z = 0.0;
            spl5->AddKnot(tmpPnt,3);
            tmpPnt.x = -500.0; tmpPnt.y = 700.0; tmpPnt.z = 0.0;
            spl5->AddKnot(tmpPnt,4);
            tmpPnt.x = 0.0; tmpPnt.y = 400.0; tmpPnt.z = 0.0;
            spl5->AddKnot(tmpPnt,5);
            tmpPnt.x =-200.0; tmpPnt.y = 0.0; tmpPnt.z = 0.0;
            spl5->AddKnot(tmpPnt,6);
            tmpPnt.x =-300.0; tmpPnt.y = 200.0; tmpPnt.z = 0.0;
            spl5->AddKnot(tmpPnt,7);
            tmpPnt.x =-300.0; tmpPnt.y = -100.0; tmpPnt.z = 0.0;
            spl5->AddKnot(tmpPnt,8);
            spl5->Close();
            
            sgCSpline* spl5_obj = sgCreateSpline(*spl5);
            SG_SPLINE::Delete(spl5);
            sgGetScene()->AttachObject(spl5_obj);
            
            sgC2DObject*  holes[3];
            
            SG_CIRCLE   cirGeo;
            SG_POINT    cirC = {-320, 400.0, 0.0};
            cirGeo.FromCenterRadiusNormal(cirC,80, zAxe);
            holes[0] = sgCreateCircle(cirGeo);
            sgGetScene()->AttachObject(holes[0]);
            
            cirC.x = -240; cirC.y = 200.0;
            cirGeo.FromCenterRadiusNormal(cirC,20, zAxe);
            holes[1] = sgCreateCircle(cirGeo);
            sgGetScene()->AttachObject(holes[1]);
            
            cirC.x = -400.0; cirC.y = 200.0;
            cirGeo.FromCenterRadiusNormal(cirC,40, zAxe);
            holes[2] = sgCreateCircle(cirGeo);
            sgGetScene()->AttachObject(holes[2]);
            
            SG_SPLINE* spl6 = SG_SPLINE::Create();
            
            tmpPnt.x = 298.0; tmpPnt.y = 200.0; tmpPnt.z = 150.0-13.0;
            spl6->AddKnot(tmpPnt,0);
            tmpPnt.x = 285.0; tmpPnt.y = 200.0; tmpPnt.z = 150.0+19.0;
            spl6->AddKnot(tmpPnt,1);
            tmpPnt.x = 243.0; tmpPnt.y = 200.0; tmpPnt.z = 150.0-31.0;
            spl6->AddKnot(tmpPnt,2);
            tmpPnt.x = 205.0; tmpPnt.y = 200.0; tmpPnt.z = 150.0-3.0;
            spl6->AddKnot(tmpPnt,3);
            tmpPnt.x = 189.0; tmpPnt.y = 200.0; tmpPnt.z = 150.0-39.0;
            spl6->AddKnot(tmpPnt,4);
            tmpPnt.x = 152.0; tmpPnt.y = 200.0; tmpPnt.z = 150.0+23.0;
            spl6->AddKnot(tmpPnt,5);
            tmpPnt.x = 75.0; tmpPnt.y = 200.0; tmpPnt.z = 150.0+23.0;
            spl6->AddKnot(tmpPnt,6);
            
            sgCSpline* spl6_obj = sgCreateSpline(*spl6);
            SG_SPLINE::Delete(spl6);
            sgGetScene()->AttachObject(spl6_obj);
            
            
            SG_SPLINE* spl7 = SG_SPLINE::Create();
            
            tmpPnt.x = 296.0; tmpPnt.y = 350.0; tmpPnt.z = 150.0+8.0;
            spl7->AddKnot(tmpPnt,0);
            tmpPnt.x = 266.0; tmpPnt.y = 350.0; tmpPnt.z = 150.0-20.0;
            spl7->AddKnot(tmpPnt,1);
            tmpPnt.x = 212.0; tmpPnt.y = 350.0; tmpPnt.z = 150.0+37.0;
            spl7->AddKnot(tmpPnt,2);
            tmpPnt.x = 100-28.0; tmpPnt.y = 350.0; tmpPnt.z = 150.0-23.0;
            spl7->AddKnot(tmpPnt,3);
            
            sgCSpline* spl7_obj = sgCreateSpline(*spl7);
            SG_SPLINE::Delete(spl7);
            sgGetScene()->AttachObject(spl7_obj);
            
            
            
            sgCLine* ln_obj = sgCreateLine(300.0, 300.0, 300.0, 79.0,300.0, 100.0);
            sgGetScene()->AttachObject(ln_obj);
            
            SG_ARC arcG;
            SG_POINT arcBeg = {298.0, 250.0 , 70.0};
            SG_POINT arcEnd = {100.0-17.0, 250.0 , 70.0};
            SG_POINT arcMid = {145.0, 250.0 , 100.0};
            arcG.FromThreePoints(arcBeg,arcEnd,arcMid,false);
            sgCArc*  arc_obj = sgCreateArc(arcG);
            sgGetScene()->AttachObject(arc_obj);
            
            
            if (tInd==4)
            {
                [self addText:@"Splines" Size:30 OnX:15 OnY:-(pageW/2-120)];
            }
            else
            {
                sgCObject* coons = sgSurfaces::Coons((const sgC2DObject&)*spl1_obj,
                                                     (const sgC2DObject&)*spl2_obj,
                                                     (const sgC2DObject&)*spl3_obj,
                                                     (const sgC2DObject*)spl4_obj,36,36,4,4);
                
                sgC3DObject* obj3D = reinterpret_cast<sgC3DObject*>(coons);
                
                [self add3D:obj3D WithColor:COLOR_RED OnPosX:0 OnPosY:0 OnPosZ:20];
                
                sgC3DObject* fcO = (sgC3DObject*)sgSurfaces::Face((const sgC2DObject&)(*spl5_obj),
                                                                  (const sgC2DObject**)(&holes[0]),3);
                
                [self add3D:fcO WithColor:COLOR_BLUE OnPosX:0 OnPosY:0 OnPosZ:20];
                
                
                sgC2DObject*  objcts[4];
                objcts[0] = spl6_obj;
                objcts[1] = arc_obj;
                objcts[2] = ln_obj;
                objcts[3] = spl7_obj;
                
                double param[4];
                param[0] = param[1] = param[2] = param[3] = 0.0;
                
                sgC3DObject* surf = (sgC3DObject*)sgSurfaces::SplineSurfaceFromSections((const sgC2DObject**)(objcts),
                                                                                        param,4,false);
                
                [self add3D:surf WithColor:COLOR_WHITE OnPosX:0 OnPosY:0 OnPosZ:-20];
                
                
                [self addText:@"Surfaces and Solids" Size:40 OnX:-250 OnY:-(pageW/2-130)];
                [self addText:@"from" Size:40 OnX:-100 OnY:-(pageW/2-80)];
                [self addText:@"Contours" Size:40 OnX:-150 OnY:-(pageW/2-30)];
                
                
            }
            
        }
            break;
        case 6:
        {
            
            SG_POINT  tmpPnt;
            SG_SPLINE* spl8 = SG_SPLINE::Create();
            int fl=0;
            for (double i=0.0;i<2.0*3.14159265;i+=0.4)
            {
                tmpPnt.x = ((double)(fl%3+2)*25)*cos(i);
                tmpPnt.y = ((double)(fl%3+2)*25)*sin(i);
                tmpPnt.z = 100.0;
                spl8->AddKnot(tmpPnt,fl);
                fl++;
            }
            spl8->Close();
            
            sgCSpline* spl8_obj = sgCreateSpline(*spl8);
            SG_SPLINE::Delete(spl8);
            sgGetScene()->AttachObject(spl8_obj);
            
            
            SG_SPLINE* spl9 = SG_SPLINE::Create();
            
            tmpPnt.x = 0.0; tmpPnt.y = -22; tmpPnt.z = 3.0;
            spl9->AddKnot(tmpPnt,0);
            tmpPnt.x = 34; tmpPnt.y = 22; tmpPnt.z = 3.0;
            spl9->AddKnot(tmpPnt,1);
            tmpPnt.x = -40; tmpPnt.y = 15; tmpPnt.z = 3.0;
            spl9->AddKnot(tmpPnt,2);
            tmpPnt.x = -30; tmpPnt.y = -40; tmpPnt.z = 3.0;
            spl9->AddKnot(tmpPnt,3);
            spl9->Close();
            
            
            sgCSpline* spl9_obj = sgCreateSpline(*spl9);
            SG_SPLINE::Delete(spl9);
            sgGetScene()->AttachObject(spl9_obj);
            
            sgC2DObject* ooo[3];
            ooo[1] = spl8_obj;
            ooo[0] = spl9_obj;
            
            
            ooo[2] = (sgC2DObject*)spl9_obj->Clone();
            sgGetScene()->AttachObject(ooo[2]);
            
            SG_POINT zP = {0, 0, 3};
            ooo[2]->InitTempMatrix();//->Rotate(zeroP,xAxe, -M_PI/2.0);
            SG_VECTOR trV = {0, 0.0, 250.0};
            ooo[2]->GetTempMatrix()->Translate(trV);
            ooo[2]->ApplyTempMatrix();
            ooo[2]->DestroyTempMatrix();
            
            double param[3];
            param[0] = 0.1;
            param[1] = 0.0;
            param[2] = 0.5;
            
            
            sgC3DObject* lsf = (sgC3DObject*)sgSurfaces::SplineSurfaceFromSections((const sgC2DObject**)(&ooo[0]),
                                                                                   param,3,true);
            
            [self add3D:lsf WithColor:COLOR_YELLOW OnPosX:-150 OnPosY:200 OnPosZ:0];
            
            
            sgC3DObject* linO1 = (sgC3DObject*)sgSurfaces::LinearSurfaceFromSections((const sgC2DObject&)(*spl8_obj),
                                                                                     (const sgC2DObject&)(*(ooo[2])),0.0,false);
            
            
            [self add3D:linO1 WithColor:COLOR_RED OnPosX:150 OnPosY:200 OnPosZ:0];
            
            [self addText:@"Surfaces and Solids" Size:40 OnX:-250 OnY:-(pageW/2-130)];
            [self addText:@"from" Size:40 OnX:-100 OnY:-(pageW/2-80)];
            [self addText:@"Contours" Size:40 OnX:-150 OnY:-(pageW/2-30)];
        }
            break;
        case 7:
        {
            SG_CIRCLE   cirGeo1;
            SG_POINT    cirC1 = {-232.0, -2.0, 0.0};
            cirGeo1.FromCenterRadiusNormal(cirC1,30.0, zAxe);
            sgC2DObject*  cir1 = sgCreateCircle(cirGeo1);
            sgGetScene()->AttachObject(cir1);
            
            SG_CIRCLE   cirGeo2;
            SG_POINT    cirC2 = {-232.0, -7.3, 0.0};
            cirGeo2.FromCenterRadiusNormal(cirC2,15, zAxe);
            sgC2DObject*  cir2 = sgCreateCircle(cirGeo2);
            sgGetScene()->AttachObject(cir2);
            
            SG_POINT axeP1 = {-170.0, -30.0, 0.0};
            SG_POINT axeP2 = {-170.0, 30.0, 0.0};
            
            
            sgC3DObject* spirO = (sgC3DObject*)sgKinematic::Spiral((const sgC2DObject&)(*cir1),
                                                                   (const sgC2DObject**)(&cir2),1,
                                                                   axeP1,axeP2,90,530,24,true);
            
            if (spirO)
            {
                spirO->InitTempMatrix()->Rotate(axeP1, yAxe, M_PI);
                spirO->ApplyTempMatrix();
                spirO->DestroyTempMatrix();
            }
            [self add3D:spirO WithColor:COLOR_YELLOW OnPosX:0 OnPosY:7 OnPosZ:100];
            
            {
                sgCObject*   objcts[7];
                
                SG_POINT    ArP1;
                SG_POINT    ArP2;
                SG_POINT    ArP3;
                SG_ARC      ArcGeo;
                
                ArP1.x = 0.0; ArP1.y = -200.0; ArP1.z = 0.0;
                ArP2.x = 100.0; ArP2.y = -100.0; ArP2.z = 0.0;
                ArP3.x = 40; ArP3.y = -120; ArP3.z = 0.0;
                ArcGeo.FromThreePoints(ArP1,ArP2,ArP3,false);
                objcts[0] = sgCreateArc(ArcGeo);
                
                ArP1.x = 100.0; ArP1.y = -100.0; ArP1.z = 0.0;
                ArP2.x = 200.0; ArP2.y = 0.0; ArP2.z = 0.0;
                ArP3.x = 190; ArP3.y = -50; ArP3.z = 0.0;
                ArcGeo.FromThreePoints(ArP1,ArP2,ArP3,false);
                objcts[1] = sgCreateArc(ArcGeo);
                
                ArP1.x = 200.0; ArP1.y = 0.0; ArP1.z = 0.0;
                ArP2.x = 100.0; ArP2.y = 100.0; ArP2.z = 0.0;
                ArP3.x = 160; ArP3.y = 80; ArP3.z = 0.0;
                ArcGeo.FromThreePoints(ArP1,ArP2,ArP3,false);
                objcts[2] = sgCreateArc(ArcGeo);
                
                objcts[3] = sgCreateLine(100.0, 100.0, 0.0, -100.0, 100.0, 0.0);
                
                ArP1.x = -100.0; ArP1.y = 100.0; ArP1.z = 0.0;
                ArP2.x = -100.0; ArP2.y = 0.0; ArP2.z = 100.0;
                ArP3.x = -110; ArP3.y = 100.0; ArP3.z = 0.0;
                ArcGeo.FromThreePoints(ArP1,ArP2,ArP3,false);
                objcts[4] = sgCreateArc(ArcGeo);
                
                sgCContour* cnt1 = sgCContour::CreateContour(objcts,5);
                sgGetScene()->AttachObject(cnt1);
                
                
                ArP1.x = -20; ArP1.y = -20; ArP1.z = 0.0;
                ArP2.x = -10; ArP2.y = 20; ArP2.z = 0.0;
                ArP3.x = -30; ArP3.y = 10; ArP3.z = 0.0;
                ArcGeo.FromThreePoints(ArP1,ArP2,ArP3,false);
                objcts[0] = sgCreateArc(ArcGeo);
                
                
                ArP1.x = -10; ArP1.y = 20; ArP1.z = 0.0;
                ArP2.x = 30; ArP2.y = 50; ArP2.z = 0.0;
                ArP3.x = 20; ArP3.y = 60; ArP3.z = 0.0;
                ArcGeo.FromThreePoints(ArP1,ArP2,ArP3,false);
                objcts[1] = sgCreateArc(ArcGeo);
                
                ArP1.x = 30; ArP1.y = 50; ArP1.z = 0.0;
                ArP2.x = -20; ArP2.y = -20; ArP2.z = 0.0;
                ArP3.x = 60; ArP3.y = -40; ArP3.z = 0.0;
                ArcGeo.FromThreePoints(ArP1,ArP2,ArP3,false);
                objcts[2] = sgCreateArc(ArcGeo);
                
                sgCContour* cnt2 = sgCContour::CreateContour(objcts,3);
                sgGetScene()->AttachObject(cnt2);
                
                SG_CIRCLE   cirGeo;
                SG_POINT    cirC = {30, -10, 0.0};
                cirGeo.FromCenterRadiusNormal(cirC,31, zAxe);
                sgC2DObject*  cir = sgCreateCircle(cirGeo);
                sgGetScene()->AttachObject(cir);
                
                SG_POINT point_in_plane = {0.0,0.0,0.0};
                
                
                bool  close = true;
                sgC3DObject* pipO = (sgC3DObject*)sgKinematic::Pipe((const sgC2DObject&)(*cnt2),
                                                                    (const sgC2DObject**)(&cir),1,
                                                                    (const sgC2DObject&)(*cnt1), point_in_plane, 0.0, close);
                
                
                
                [self add3D:pipO WithColor:COLOR_RED OnPosX:200 OnPosY:200 OnPosZ:100];
                
                SG_VECTOR shftV = {100, 50, 0};
                cnt2->InitTempMatrix()->Translate(shftV);
                cnt2->ApplyTempMatrix();
                cnt2->DestroyTempMatrix();
                
                cir->InitTempMatrix()->Translate(shftV);
                cir->ApplyTempMatrix();
                cir->DestroyTempMatrix();
                
                shftV.x = 150;
                cnt1->InitTempMatrix()->Translate(shftV);
                cnt1->ApplyTempMatrix();
                cnt1->DestroyTempMatrix();
                
                
            }
            
            [self addText:@"Spirals and Pipes" Size:30 OnX:-245 OnY:-(pageW/2-120)];
            
        }
            break;
        case 8:
        {
            int x_sz = 100;
            int y_sz = 100;
            SG_POINT*  pnts = new SG_POINT[x_sz*y_sz];
            
            for (int i=0;i<x_sz;i++)
                for (int j=0;j<y_sz;j++)
                {
                    sgFloat x = (i-x_sz/2-0.5)/2;
                    sgFloat y = (j-y_sz/2-0.5)/2;
                    pnts[i*y_sz+j].x = x;
                    pnts[i*y_sz+j].y = y;
                    sgFloat R = sqrt(x*x + y*y)+1;
                    pnts[i*y_sz+j].z = sin(R)/(R);
                }
            
            sgCObject* msh= sgSurfaces::Mesh(x_sz,y_sz,pnts);
            
            msh->InitTempMatrix()->Scale(10, 10, 80);
            msh->ApplyTempMatrix();
            msh->DestroyTempMatrix();
            
            delete [] pnts;
            
            [self add3D:(sgC3DObject*)msh WithColor:COLOR_RED OnPosX:0 OnPosY:200 OnPosZ:50];
            
            [self addText:@"Math Surfaces" Size:30 OnX:-200 OnY:-(pageW/2-120)];
            
        }
            break;
        case 9:
        {
            sgC3DObject*  mainObject = sgCreateCylinder(60, 250, 24);
            
            [self add3D:mainObject WithColor:COLOR_GREEN OnPosX:-350 OnPosY:0 OnPosZ:0];
            
            sgC3DObject*  Objs[5];
            
            for (int i=0;i<5;i++)
            {
                if (i<4)
                    Objs[i] = sgCreateTorus(60, 20, 24, 24);
                else
                    Objs[i] = sgCreateTorus(60, 5, 24, 24);
                
                [self add3D:Objs[i] WithColor:COLOR_RED OnPosX:(i==3)?-310:((i==1)?-390:-350) OnPosY:0 OnPosZ:50*(i+1)];
                
            }
            
            sgC3DObject*  sph = sgCreateSphere(25, 24, 24);
            [self add3D:sph WithColor:COLOR_RED OnPosX:-350 OnPosY:0 OnPosZ:250];
            
            [self addText:@"Boolean operations" Size:45 OnX:-250 OnY:-(pageW/2-20)];
            [self addText:@"Original objects" Size:20 OnX:-500 OnY:-(pageW/2-120)];
            
            //  SUB
            for (int i=0;i<5;i++)
            {
                sgCGroup* boolResult = sgBoolean::Sub(*mainObject, *Objs[i]);
                if (i>0)
                    sgDeleteObject(mainObject);
                mainObject = [self get3DAfterBooleanAndDeleteGroup:boolResult];
            }
            
            sgCGroup* boolResult = sgBoolean::Sub(*mainObject, *sph);
            sgDeleteObject(mainObject);
            mainObject = [self get3DAfterBooleanAndDeleteGroup:boolResult];
            
            [self add3D:mainObject WithColor:COLOR_BLUE OnPosX:200 OnPosY:0 OnPosZ:0];
            
            [self addText:@"Sub" Size:20 OnX:-200 OnY:-(pageW/2-120)];
            
            
            // UNION
            mainObject = (sgC3DObject*)sgGetScene()->GetObjectsList()->GetHead();
            
            for (int i=0;i<5;i++)
            {
                boolResult = sgBoolean::Union(*mainObject, *Objs[i]);
                if (i>0)
                    sgDeleteObject(mainObject);
                mainObject = [self get3DAfterBooleanAndDeleteGroup:boolResult];
            }
            
            boolResult = sgBoolean::Union(*mainObject, *sph);
            sgDeleteObject(mainObject);
            mainObject = [self get3DAfterBooleanAndDeleteGroup:boolResult];
            
            [self add3D:mainObject WithColor:COLOR_GRAY OnPosX:400 OnPosY:0 OnPosZ:0];
            
            [self addText:@"Union" Size:20 OnX:0 OnY:-(pageW/2-120)];
            
            
            // INTERSECTIONS
            mainObject = (sgC3DObject*)sgGetScene()->GetObjectsList()->GetHead();
            
            for (int i=0;i<5;i++)
            {
                boolResult = sgBoolean::Intersection(*mainObject, *Objs[i]);
                sgC3DObject* res = [self get3DAfterBooleanAndDeleteGroup:boolResult];
                [self add3D:res WithColor:COLOR_YELLOW OnPosX:600 OnPosY:0 OnPosZ:0];
                
            }
            
            boolResult = sgBoolean::Intersection(*mainObject, *sph);
            sgC3DObject* res = [self get3DAfterBooleanAndDeleteGroup:boolResult];
            [self add3D:res WithColor:COLOR_YELLOW OnPosX:600 OnPosY:0 OnPosZ:0];
            
            [self addText:@"Intersection" Size:20 OnX:170 OnY:-(pageW/2-120)];
            
            
            // INTERSECTIONS CONTOURS
            mainObject = (sgC3DObject*)sgGetScene()->GetObjectsList()->GetHead();
            
            SG_VECTOR trVec = {800, 0, 5};
            
            for (int i=0;i<5;i++)
            {
                boolResult = sgBoolean::IntersectionContour(*mainObject, *Objs[i]);
                if (boolResult)
                {
                    boolResult->InitTempMatrix()->Translate(trVec);
                    boolResult->ApplyTempMatrix();
                    boolResult->DestroyTempMatrix();
                    sgGetScene()->AttachObject(boolResult);
                }
            }
            
            boolResult = sgBoolean::IntersectionContour(*mainObject, *sph);
            if (boolResult)
            {
                boolResult->InitTempMatrix()->Translate(trVec);
                boolResult->ApplyTempMatrix();
                boolResult->DestroyTempMatrix();
                sgGetScene()->AttachObject(boolResult);
            }
            
            [self addText:@"Contours" Size:20 OnX:400 OnY:-(pageW/2-120)];
            
            
        }
            break;
            
            
        case 10:   // ROOM
        {
            sgFloat room_X_size = 350.0;
            sgFloat room_Y_size = 300.0;
            sgFloat room_Z_size = 200.0;
            sgFloat wall_thickness = 5;
            sgCObject*   objects_buffer[10];
            objects_buffer[0] = sgCreateLine(0.0, 0.0, 0.0, room_X_size, 0.0, 0.0);
            objects_buffer[1] = sgCreateLine(0.0, 0.0, 0.0, 0.0, -room_Y_size, 0.0);
            
            sgCContour*  cnt1 = sgCContour::CreateContour(objects_buffer,2);
            sgCContour*  cnt2 = (sgCContour*)cnt1->Clone();
            sgCContour*  cnt3 = (sgCContour*)cnt1->Clone();
            
            objects_buffer[0] = cnt1;
            objects_buffer[1] = sgCreateLine(0.0, -room_Y_size, 0.0,
                                             -wall_thickness, -room_Y_size, 0.0);
            objects_buffer[2] = sgCreateLine(-wall_thickness, -room_Y_size, 0.0,
                                             -wall_thickness, wall_thickness, 0.0);
            objects_buffer[3] = sgCreateLine(-wall_thickness, wall_thickness, 0.0,
                                             room_X_size, wall_thickness, 0.0);
            
            objects_buffer[4] = sgCreateLine(room_X_size, wall_thickness, 0.0,
                                             room_X_size, 0.0, 0.0);
            
            objects_buffer[5] = sgCContour::CreateContour(objects_buffer,5);
            
            SG_VECTOR  extVec = {0.0, 0.0, room_Z_size};
            
            
            sgCObject* walls = sgKinematic::Extrude((const sgC2DObject&)(*objects_buffer[5]),NULL,0,extVec,true);
            
            sgC3DObject* obj3D = reinterpret_cast<sgC3DObject*>(walls);
            
            [self add3D:obj3D WithColor:COLOR_GRAY OnPosX:-100 OnPosY:0 OnPosZ:0.05];
            
            int i;
            sgDeleteObject(objects_buffer[5]);
            
            objects_buffer[0] = cnt2;
            objects_buffer[1] = sgCreateLine(room_X_size, 0.0 , 0.0,
                                             room_X_size, -room_Y_size, 0.0);
            objects_buffer[2] = sgCreateLine(room_X_size, -room_Y_size, 0.0,
                                             0.0 , -room_Y_size, 0.0);
            
            objects_buffer[3] = sgCContour::CreateContour(objects_buffer,3);
            
            sgCObject* floor = sgSurfaces::Face((const sgC2DObject&)(*objects_buffer[3]),NULL,0);
            
            [self add3D:(sgC3DObject*)floor WithColor:COLOR_BLUE OnPosX:-100 OnPosY:0 OnPosZ:0.05];
            
            
            sgDeleteObject(objects_buffer[3]);
            
            sgFloat scaleCoeff = 30.0f;  // for old SDK
            
            objects_buffer[0] = sgCreateLine(0.0*scaleCoeff, 0.0*scaleCoeff, 0.0*scaleCoeff,
                                             0.0*scaleCoeff, -0.3*scaleCoeff, 0.0*scaleCoeff);
            
            SG_ARC arcGeo;
            SG_POINT p1,p2,p3;
            p1.x = 0.0*scaleCoeff;    p1.y = -0.3*scaleCoeff;   p1.z = 0.0*scaleCoeff;
            p2.x = -0.104*scaleCoeff; p2.y = -0.193*scaleCoeff; p2.z = 0.0*scaleCoeff;
            p3.x = -0.099*scaleCoeff; p3.y = -0.232*scaleCoeff; p3.z = 0.0*scaleCoeff;
            arcGeo.FromThreePoints(p1, p2, p3,false);
            objects_buffer[1] = sgCreateArc(arcGeo);
            
            p1.x = -0.104*scaleCoeff; p1.y = -0.193*scaleCoeff; p1.z = 0.0*scaleCoeff;
            p2.x = -0.205*scaleCoeff; p2.y = -0.095*scaleCoeff; p2.z = 0.0*scaleCoeff;
            p3.x = -0.132*scaleCoeff; p3.y = -0.128*scaleCoeff; p3.z = 0.0*scaleCoeff;
            arcGeo.FromThreePoints(p1, p2, p3,false);
            objects_buffer[2] = sgCreateArc(arcGeo);
            
            p1.x = -0.205*scaleCoeff; p1.y = -0.095*scaleCoeff; p1.z = 0.0*scaleCoeff;
            p2.x = -0.300*scaleCoeff; p2.y = 0.000*scaleCoeff;  p2.z = 0.0*scaleCoeff;
            p3.x = -0.294*scaleCoeff; p3.y = -0.047*scaleCoeff; p3.z = 0.0*scaleCoeff;
            arcGeo.FromThreePoints(p1, p2, p3,false);
            objects_buffer[3] = sgCreateArc(arcGeo);
            
            objects_buffer[4] = sgCreateLine(-0.3*scaleCoeff, 0.0*scaleCoeff*scaleCoeff, 0.0,
                                             0.0*scaleCoeff, 0.0*scaleCoeff, 0.0*scaleCoeff);
            
            objects_buffer[5] = sgCContour::CreateContour(objects_buffer,5);
            
            SG_POINT  pntOnPl = {0.0, 0.0, 0.0};
            bool      clPl = true;
            
            sgCObject* down_plinth = sgKinematic::Pipe((const sgC2DObject&)(*objects_buffer[5]),NULL,0,
                                                       (const sgC2DObject&)(*cnt3),pntOnPl, 90.0, clPl);
            
            SG_VECTOR transV;
            
            [self add3D:(sgC3DObject*)down_plinth WithColor:COLOR_RED OnPosX:-100 OnPosY:0 OnPosZ:0.01];
            
            
            sgDeleteObject(objects_buffer[5]);
            sgDeleteObject(cnt3);
            
            
            SG_SPLINE*  spl = SG_SPLINE::Create();
            p1.x = 0.1*scaleCoeff; p1.y = 0.0*scaleCoeff; p1.z = 4.0*scaleCoeff;
            spl->AddKnot(p1,0);
            p1.x = 0.2*scaleCoeff; p1.y = 0.0*scaleCoeff; p1.z = 2.0*scaleCoeff;
            spl->AddKnot(p1,1);
            p1.x = 0.03*scaleCoeff; p1.y = 0.0*scaleCoeff; p1.z = 1.8*scaleCoeff;
            spl->AddKnot(p1,2);
            p1.x = 0.1*scaleCoeff; p1.y = 0.0*scaleCoeff; p1.z = 1.0*scaleCoeff;
            spl->AddKnot(p1,3);
            p1.x = 0.05*scaleCoeff; p1.y = 0.0*scaleCoeff; p1.z = 0.1*scaleCoeff;
            spl->AddKnot(p1,4);
            p1.x = 0.6*scaleCoeff; p1.y = 0.0*scaleCoeff; p1.z = 0.01*scaleCoeff;
            spl->AddKnot(p1,5);
            
            objects_buffer[0] = sgCreateSpline(*spl);
            SG_SPLINE::Delete(spl);
            p1.x = p1.y = p1.z = 0.0;
            p2.x = p2.y = 0.0; p2.z = 10.0;
            sgCObject* stem = sgKinematic::Rotation(*(sgC2DObject*)objects_buffer[0],p1,p2,360,false);
            
            SG_VECTOR transV1 = {1.0*scaleCoeff, -1.0*scaleCoeff ,0.0*scaleCoeff};
            stem->InitTempMatrix()->Translate(transV1);
            stem->ApplyTempMatrix();
            stem->DestroyTempMatrix();
            
            sgDeleteObject(objects_buffer[0]);
            
            [self add3D:(sgC3DObject*)stem WithColor:COLOR_YELLOW OnPosX:-100 OnPosY:0 OnPosZ:0];
            
            
            
            
            objects_buffer[0] = sgCreateLine(1.0*scaleCoeff, 0.0*scaleCoeff, 3.7*scaleCoeff,
                                             0.4*scaleCoeff, 0.0*scaleCoeff, 5.0*scaleCoeff);
            sgCObject* lamp_head = sgKinematic::Rotation(*(sgC2DObject*)objects_buffer[0],p1,p2,360,false);
            lamp_head->InitTempMatrix()->Translate(transV1);
            lamp_head->ApplyTempMatrix();
            lamp_head->DestroyTempMatrix();
            [self add3D:(sgC3DObject*)lamp_head WithColor:COLOR_GREEN OnPosX:-100 OnPosY:0 OnPosZ:0];
            
            sgDeleteObject(objects_buffer[0]);
            
            objects_buffer[0] = sgCreateTorus(1.0*scaleCoeff, 0.05*scaleCoeff, 24,24);
            transV1.z = 3.7*scaleCoeff;
            objects_buffer[0]->InitTempMatrix()->Translate(transV1);
            objects_buffer[0]->ApplyTempMatrix();
            objects_buffer[0]->DestroyTempMatrix();
            [self add3D:(sgC3DObject*)objects_buffer[0] WithColor:COLOR_RED OnPosX:-100 OnPosY:0 OnPosZ:0];
            
            
            
            objects_buffer[0] = sgCreateTorus(0.4*scaleCoeff, 0.05*scaleCoeff, 24,24);
            transV1.z = 5.0*scaleCoeff;
            objects_buffer[0]->InitTempMatrix()->Translate(transV1);
            objects_buffer[0]->ApplyTempMatrix();
            objects_buffer[0]->DestroyTempMatrix();
            [self add3D:(sgC3DObject*)objects_buffer[0] WithColor:COLOR_RED OnPosX:-100 OnPosY:0 OnPosZ:0];
            
            
            SG_CIRCLE tableLeg;
            tableLeg.center.x = 0.0;tableLeg.center.y = 0.0;tableLeg.center.z = 0.0;
            tableLeg.normal.x = 0.0;tableLeg.normal.y = 0.0;tableLeg.normal.z = 1.0;
            tableLeg.radius = 0.04*scaleCoeff;
            objects_buffer[0] = sgCreateCircle(tableLeg);
            
            extVec.x = 0.2*scaleCoeff;extVec.y = 0.2*scaleCoeff;extVec.z = 2.5*scaleCoeff;
            sgCObject* table_legs[4];
            
            table_legs[0] = sgKinematic::Extrude((const sgC2DObject&)(*objects_buffer[0]),NULL,0,extVec,true);
            extVec.x = -0.2*scaleCoeff;extVec.y = 0.2*scaleCoeff;extVec.z = 2.5*scaleCoeff;
            table_legs[1] = sgKinematic::Extrude((const sgC2DObject&)(*objects_buffer[0]),NULL,0,extVec,true);
            extVec.x = -0.2*scaleCoeff;extVec.y = -0.2*scaleCoeff;extVec.z = 2.5*scaleCoeff;
            table_legs[2] = sgKinematic::Extrude((const sgC2DObject&)(*objects_buffer[0]),NULL,0,extVec,true);
            extVec.x = 0.2*scaleCoeff;extVec.y = -0.2*scaleCoeff;extVec.z = 2.5*scaleCoeff;
            table_legs[3] = sgKinematic::Extrude((const sgC2DObject&)(*objects_buffer[0]),NULL,0,extVec,true);
            
            sgDeleteObject(objects_buffer[0]);
            
            for (i=0;i<4;i++)
            {
                transV1.z = 0.0;
                switch(i)
                {
                    case 0:
                        transV1.x = -2.0*scaleCoeff;
                        transV1.y = -1.0*scaleCoeff;
                        break;
                    case 1:
                        transV1.x = 2.0*scaleCoeff;
                        transV1.y = -1.0*scaleCoeff;
                        break;
                    case 2:
                        transV1.x = 2.0*scaleCoeff;
                        transV1.y = 1.0*scaleCoeff;
                        break;
                    case 3:
                        transV1.x = -2.0*scaleCoeff;
                        transV1.y = 1.0*scaleCoeff;
                        break;
                }
                table_legs[i]->InitTempMatrix()->Translate(transV1);
                table_legs[i]->ApplyTempMatrix();
                table_legs[i]->DestroyTempMatrix();
                objects_buffer[i] = table_legs[i];
            }
            
            objects_buffer[4] = sgCreateBox(4.4*scaleCoeff,2.4*scaleCoeff,0.1*scaleCoeff);
            
            transV1.x = -2.2*scaleCoeff;
            transV1.y = -1.2*scaleCoeff;
            transV1.z = 2.5*scaleCoeff;
            objects_buffer[4]->InitTempMatrix()->Translate(transV1);
            objects_buffer[4]->ApplyTempMatrix();
            objects_buffer[4]->DestroyTempMatrix();
            
            transV1.x = 8.0*scaleCoeff;
            transV1.y = -4.0*scaleCoeff;
            transV1.z = 0.01*scaleCoeff;
            for (int i=0; i<5;i++)
            {
                [self add3D:(sgC3DObject*)objects_buffer[i] WithColor:COLOR_WHITE OnPosX:transV1.x-100 OnPosY:transV1.y OnPosZ:transV1.z];
            }
            
        }
            break;
        case 11:
        {
            
            NSString* pathSTL = [[NSBundle mainBundle] pathForResource:@"dino" ofType:@"stl"];
            sgFileManager::ImportSTL(sgGetScene(), [pathSTL cStringUsingEncoding:NSASCIIStringEncoding] );
            
        }
            break;
        case 12:
        {
            
            NSString* pathDXF = [[NSBundle mainBundle] pathForResource:@"statue" ofType:@"dxf"];
            sgFileManager::ImportDXF(sgGetScene(), [pathDXF cStringUsingEncoding:NSASCIIStringEncoding] );
            
        }
            break;
        default:
        {
            [self add3D:sgCreateBox(90,80,50) WithColor:COLOR_BLUE OnPosX:-300 OnPosY:200 OnPosZ:20];
            [self add3D:sgCreateSphere(50,24,24) WithColor:COLOR_YELLOW OnPosX:-100 OnPosY:220 OnPosZ:50];
            [self add3D:sgCreateCone(50,0, 120, 24) WithColor:COLOR_RED OnPosX:40 OnPosY:220 OnPosZ:20];
            [self add3D:sgCreateCylinder(50, 120, 24) WithColor:COLOR_GREEN OnPosX:180 OnPosY:220 OnPosZ:20];
            
            [self add3D:sgCreateTorus(80, 40, 36, 36) WithColor:COLOR_GRAY OnPosX:-250 OnPosY:0 OnPosZ:20];
            [self add3D:sgCreateEllipsoid(80, 60, 30, 36, 36) WithColor:COLOR_WHITE OnPosX:0 OnPosY:0 OnPosZ:50];
            [self add3D:sgCreateSphericBand(70, -0.2, 0.6, 36) WithColor:COLOR_YELLOW OnPosX:180 OnPosY:0 OnPosZ:50];
            
            
            
            [self addText:@"3D Primitives" Size:30 OnX:15 OnY:-(pageW/2-120)];
            
        }
            break;
    }
}





@end
