#import "sgObjectNode.h"


void   GetObjectColor(int colIndex, float& colRed, float&colGreen, float& colBlue)
{
    const  SG_POINT  colorsOldVersion[] =
    {
        {0.909804, 0.129412, 0.176471},
        {0.780392, 0.188235, 0.180392},
        {0.921569, 0.278431, 0.184314},
        {0.941177, 0.419608, 0.184314},
        {0.980392, 0.721569, 0.188235},
        {0.996078, 0.937255, 0.207843},
        {0.611765, 0.784314, 0.274510},
        {0.298039, 0.701961, 0.309804},
        {0.074510, 0.568627, 0.290196},
        {0.098039, 0.392157, 0.149020},
        {0.035294, 0.368627, 0.317647},
        {0.054902, 0.439216, 0.490196},
        {0.176471, 0.564706, 0.760784},
        {0.027451, 0.298039, 0.450980},
        {0.054902, 0.180392, 0.376471},
        {0.313726, 0.141176, 0.407843},
        {0.682353, 0.219608, 0.568627},
        {0.847059, 0.156863, 0.513726},
        {0.913726, 0.411765, 0.435294},
        {0.945098, 0.564706, 0.376471},
        {0.984314, 0.941177, 0.545098},
        {0.450980, 0.760784, 0.572549},
        {0.490196, 0.796079, 0.941177},
        {0.729412, 0.572549, 0.756863},
        {0.917647, 0.670588, 0.686275},
        {1.000000, 1.000000, 1.000000},
        {0.800000, 0.807843, 0.811765},
        {0.650980, 0.658824, 0.670588},
        {0.517647, 0.525490, 0.529412},
        {0.364706, 0.368627, 0.380392},
        {0.290196, 0.290196, 0.298039},
        {0.137255, 0.121569, 0.125490},
        {0.298039, 0.250980, 0.105882},
        {0.333333, 0.200000, 0.113725},
        {0.431373, 0.254902, 0.113725},
        {0.611765, 0.400000, 0.180392},
        {0.588235, 0.435294, 0.192157},
        {0.686275, 0.533333, 0.360784}
    };
    
    if (colIndex<38)
    {
        colRed = colorsOldVersion[colIndex].x;
        colGreen = colorsOldVersion[colIndex].y;
        colBlue = colorsOldVersion[colIndex].z;
        return;
    }
}

@implementation sgObjectNode
{
    sgCObject** object;
    BOOL        wasSuccessUpdated;
}

-(id) initWithObject:(sgCObject**)obj
{
    if(self = [super init])
    {
        object = obj;
        wasSuccessUpdated = NO;
        
        [self updatePtr];
    }
    return self;
}


-(int)         sg_get_points_count:(sgCObject*)sgObj
{
    switch (sgObj->GetType()) {
        case SG_OT_POINT:
            return 1;
        case SG_OT_LINE:
            return 2;
        case SG_OT_CIRCLE:
        {
            sgCCircle* circ = (sgCCircle*)sgObj;
            return circ->GetPointsCount();
        }
        case SG_OT_ARC:
        {
            sgCArc* ar = (sgCArc*)sgObj;
            return ar->GetPointsCount();
        }
        case SG_OT_SPLINE:
        {
            sgCSpline* spl = (sgCSpline*)sgObj;
            return spl->GetGeometry()->GetPointsCount();
        }
        case SG_OT_TEXT:
        {
            sgCText* tx = (sgCText*)sgObj;
            return tx->GetLinesCount()*2;
        }
        case SG_OT_DIM:
        {
            sgCDimensions* dm = (sgCDimensions*)sgObj;
            return dm->GetLinesCount()*2;
        }
    }
    return 0;
}
-(sgFloat*)   sg_get_points:(sgCObject*)sgObj
{
    switch (sgObj->GetType()) {
        case SG_OT_POINT:
        {
            sgCPoint* pp = (sgCPoint*)sgObj;
            return ( sgFloat*)(pp->GetGeometry());
        }
        case SG_OT_LINE:
        {
            sgCLine* ln = (sgCLine*)sgObj;
            return ( sgFloat*)(ln->GetGeometry());
        }
        case SG_OT_CIRCLE:
        {
            sgCCircle* circ = (sgCCircle*)sgObj;
            return ( sgFloat*)(circ->GetPoints());
        }
        case SG_OT_ARC:
        {
            sgCArc* ar = (sgCArc*)sgObj;
            return ( sgFloat*)(ar->GetPoints());
        }
        case SG_OT_SPLINE:
        {
            sgCSpline* spl = (sgCSpline*)sgObj;
            return ( sgFloat*)(spl->GetGeometry()->GetPoints());
        }
        case SG_OT_TEXT:
        {
            sgCText* tx = (sgCText*)sgObj;
            return ( sgFloat*)(tx->GetLines());
        }
        case SG_OT_DIM:
        {
            sgCDimensions* dm = (sgCDimensions*)sgObj;
            return ( sgFloat*)(dm->GetLines());
        }
    }
    return 0;
}


-(BOOL)        sg_is_close:(sgCObject*)obj
{
    sgCObject* sgObj = (sgCObject*)obj;
    if (sgObj->GetType()==SG_OT_CIRCLE)
        return YES;
    return NO;
}

-(BOOL)        sg_is_strip:(sgCObject*)obj
{
    sgCObject* sgObj = (sgCObject*)obj;
    if (sgObj->GetType()==SG_OT_CIRCLE || sgObj->GetType()==SG_OT_ARC || sgObj->GetType()==SG_OT_SPLINE)
        return YES;
    return NO;
}


-(bool)  updatePtr
{
    if (!object || wasSuccessUpdated)
        return false;
    sgCObject* obj = *object;
    if (!obj)
        return false;
    
    if (obj->GetType()==SG_OT_3D)
    {
        sgC3DObject* cur3D = (sgC3DObject*)obj;
        if (cur3D->GetTriangles()==NULL)
        {
            cur3D->Triangulate(SG_DELAUNAY_TRIANGULATION, SG_SEPARATE_BUFFERS);
        }
        if (cur3D->GetTriangles()==NULL)
        {
            NSLog(@"Error - can not triangulate object");
            return false;
        }
        int nTr = cur3D->GetTriangles()->nTr;
        const SG_ALL_TRIANGLES* tr = cur3D->GetTriangles();
        
        NSArray* geoArr = [NSArray arrayWithObjects:
                           [SCNGeometrySource geometrySourceWithVertices:(SCNVector3*)(tr->allVertex) count:nTr*3],
                           [SCNGeometrySource geometrySourceWithNormals:(SCNVector3 *)(tr->allNormals) count:nTr*3],
                           nil];
        
        int* indxs = (int*)malloc(3*nTr*sizeof(int));
        
        for (int i=0;i<3*nTr;i++)
            indxs[i] = i;
        
        NSArray* geoEll = [NSArray arrayWithObjects:
                           [SCNGeometryElement geometryElementWithData:[NSData dataWithBytes:indxs length:(3*nTr*sizeof(int))] primitiveType:SCNGeometryPrimitiveTypeTriangles primitiveCount:nTr bytesPerIndex:sizeof(int)],
                           nil];
        
        
        self.geometry = [SCNGeometry geometryWithSources:geoArr elements:geoEll];
        
        free(indxs);
        
        cur3D->FreeTriangles();
        
        float colR,colG,colB;
        GetObjectColor(cur3D->GetAttribute(SG_OA_COLOR), colR, colG, colB);
        //use the specified images named as the texture
        self.geometry.firstMaterial.diffuse.contents = [NSColor colorWithRed:colR green:colG blue:colB alpha:1.0];
        self.geometry.firstMaterial.specular.contents = [NSColor whiteColor];

        if (cur3D->Get3DObjectType()==SG_SURFACE)
            self.geometry.firstMaterial.doubleSided = YES;

        const double* dat = cur3D->GetWorldMatrixData();
        GLKMatrix4 matr;
        
        float  tmpFloats[16];
        for (int flIndx = 0; flIndx<16; flIndx++)
            tmpFloats[flIndx]=dat[flIndx];
        memcpy(&matr, tmpFloats, sizeof(matr));
        
        self.transform = SCNMatrix4Mult(SCNMatrix4FromGLKMatrix4( matr ), SCNMatrix4MakeRotation(-3.14159265/2, 1, 0, 0));
        
        wasSuccessUpdated = YES;
       // sgSetToObjectTreeNodeHandle(obj, (__bridge ISceneTreeControl::TREENODEHANDLE)self);
        
    }
    else if (obj->GetType()==SG_OT_POINT || obj->GetType()==SG_OT_LINE || obj->GetType()==SG_OT_CIRCLE ||
             obj->GetType()==SG_OT_ARC || obj->GetType()==SG_OT_SPLINE ||
             obj->GetType()==SG_OT_TEXT || obj->GetType()==SG_OT_DIM)
    {
        self.position = SCNVector3Make(0, 1, 0);
        self.rotation = SCNVector4Make(1, 0, 0, -3.14159265/2);
        
        
        int  nPnts = [self sg_get_points_count:obj];
        
        sgFloat* vert_pointer = [self sg_get_points:obj ];
        
        SCNGeometrySource* geom = [SCNGeometrySource geometrySourceWithVertices:(SCNVector3*)vert_pointer count:nPnts];
        
        NSArray* geoArr = [NSArray arrayWithObjects:geom,nil];
        
        int cnt = 0;
        if ([self sg_is_strip:obj])
        {
            cnt = 2*nPnts;
        }
        else
        {
            cnt = nPnts;
        }
        
        int subParam = 0;
        if ([self sg_is_close:obj])
        {
            subParam = -1;
        }
        
        int stripNonCloseParam = 0;
        if (!([self sg_is_close:obj]) && ([self sg_is_strip:obj]))
        {
            stripNonCloseParam = -2;
        }
        
        int* indxs = (int*)malloc((cnt+stripNonCloseParam)*sizeof(int));
        
        for (int i=0;i<(cnt+stripNonCloseParam);i++)
            indxs[i] = i;
        
        for (int i=0;i<(cnt+subParam+stripNonCloseParam);i++)
        {
            if ([self sg_is_strip:obj])
            {
                int reminder = i%2;
                if (reminder==1)
                {
                    indxs[i] = i/2;
                }
                else
                {
                    indxs[i] = i/2+1;
                }
            }
            else
            {
                indxs[i] = i;
            }
        }
        
        SCNGeometryElement* ells = [SCNGeometryElement geometryElementWithData:[NSData dataWithBytes:indxs length:(cnt+stripNonCloseParam)*sizeof(int)] primitiveType:SCNGeometryPrimitiveTypeLine primitiveCount:(cnt-1+stripNonCloseParam)/2+1 bytesPerIndex:sizeof(int)];
        
        
        
        NSArray* geoEll = [NSArray arrayWithObjects:ells, nil];
        
        
        //attach a box of 5x5x5
        self.geometry = [SCNGeometry geometryWithSources:geoArr elements:geoEll];
        
        free(indxs);

       
    }
    
    return true;

}

-(void)  createObjectFromNode:(SCNNode*)draggableObject
{
    // Get the vertex sources
    NSArray *vertexSources = [draggableObject.geometry geometrySourcesForSemantic:SCNGeometrySourceSemanticVertex];
    
    // Get the first source
    SCNGeometrySource *vertexSource = vertexSources[0]; // TODO: Parse all the sources
    
    NSInteger stride = vertexSource.dataStride; // in bytes
    NSInteger offset = vertexSource.dataOffset; // in bytes
    
    NSInteger componentsPerVector = vertexSource.componentsPerVector;
    NSInteger bytesPerVector = componentsPerVector * vertexSource.bytesPerComponent;
    NSInteger vectorCount = vertexSource.vectorCount;
    
    SCNVector3 vertices[vectorCount]; // A new array for vertices
    
    // for each vector, read the bytes
    for (NSInteger i=0; i<vectorCount; i++) {
        
        // Assuming that bytes per component is 4 (a float)
        // If it was 8 then it would be a double (aka CGFloat)
        float vectorData[componentsPerVector];
        
        // The range of bytes for this vector
        NSRange byteRange = NSMakeRange(i*stride + offset, // Start at current stride + offset
                                        bytesPerVector);   // and read the lenght of one vector
        
        // Read into the vector data buffer
        [vertexSource.data getBytes:&vectorData range:byteRange];
        
        // At this point you can read the data from the float array
        float x = vectorData[0];
        float y = vectorData[1];
        float z = vectorData[2];
        
        // ... Maybe even save it as an SCNVector3 for later use ...
        vertices[i] = SCNVector3Make(x, y, z);
        
        // ... or just log it
        //NSLog(@"x:%f, y:%f, z:%f", x, y, z);
    }
    
    SCNGeometryElement *element = [draggableObject.geometry geometryElementAtIndex:0];
    const void *inputPrimitive = [element.data bytes];
    NSUInteger primitiveCount = element.primitiveCount;
    
    size_t bpi = element.bytesPerIndex;
    
    SG_INDEX_TRIANGLE* indexes = (SG_INDEX_TRIANGLE*)malloc(primitiveCount*sizeof(SG_INDEX_TRIANGLE));
    if (bpi==2)
    {
        for (int i=0;i<primitiveCount;i++)
        {
            indexes[i].ver_indexes[0] = (long)(*((short*)inputPrimitive+i*3));
            indexes[i].ver_indexes[1] = (long)(*((short*)inputPrimitive+(i*3+1)));
            indexes[i].ver_indexes[2] = (long)(*((short*)inputPrimitive+(i*3+2)));
            //NSLog(@"%li - %li - %li", indexes[i].ver_indexes[0], indexes[i].ver_indexes[1], indexes[i].ver_indexes[2]);
        }
    }
    else if (bpi==4)
    {
        for (int i=0;i<primitiveCount;i++)
        {
            indexes[i].ver_indexes[0] = (long)(*((int*)inputPrimitive+i*3));
            indexes[i].ver_indexes[1] = (long)(*((int*)inputPrimitive+(i*3+1)));
            indexes[i].ver_indexes[2] = (long)(*((int*)inputPrimitive+(i*3+2)));
            //NSLog(@"%li - %li - %li", indexes[i].ver_indexes[0], indexes[i].ver_indexes[1], indexes[i].ver_indexes[2]);
        }
    }
    
    sgCObject* obj = sgFileManager::ObjectFromTriangles((const SG_POINT*)&vertices, vectorCount, indexes, primitiveCount, true);
    
    if (obj->GetType()==SG_OT_3D)
    {
        sgC3DObject* ooo = (sgC3DObject*)obj;
        if (ooo->Get3DObjectType()==SG_BODY)
            NSLog(@"SOLID!!!! :) ");
        if (ooo->Get3DObjectType()==SG_SURFACE)
            NSLog(@"SURFACE :( ");
    }
    if (obj)
        sgGetScene()->AttachObject(obj);
    
    free(indexes);

}

@end
