#include "Scenes.h"
#include <math.h>
#include <vector>


#include <cstdlib>

#import "../../sgCore/sgCore.h"



void OnBnClickedPrPoints()
{
  sgGetScene()->Clear();

  for (int i=0;i<30;i++)
  {
    for (int j=0;j<30;j++)
      {
        double pZ = 0.05*(i-15)*(i-15)+0.03*(j-15)*(j-15);
        sgCPoint* pnt = sgCreatePoint(i-15, j-15, pZ);
        sgGetScene()->AttachObject(pnt);
        pnt->SetAttribute(SG_OA_COLOR,((int)pZ)%100);
        pnt->SetAttribute(SG_OA_LINE_THICKNESS, ((int)pZ)%3+1);
      }
  }
}

void OnBnClickedPrLines()
{
  sgGetScene()->Clear();

  for (double i=0.0;i<2.0*3.14159265;i+=0.1)
  {
      {
        sgCLine* ln = sgCreateLine(0, 0, 0.0, 5*cos(i), 5*sin(i), 0.0);
        sgGetScene()->AttachObject(ln);
        ln->SetAttribute(SG_OA_COLOR,((int)(10*i))%200);
        ln->SetAttribute(SG_OA_LINE_THICKNESS, ((int)(10*i))%5);
        ln->SetAttribute(SG_OA_LINE_TYPE, ((int)(10*i))%5);
      }
  }
}

void OnBnClickedPrCircles()
{
  sgGetScene()->Clear();

  for (double i=0.0;i<2.0*3.14159265;i+=0.4)
  {
    SG_POINT   crCen = {5.0*cos(i),5.0*sin(i),0.0};
    SG_VECTOR  crNor;
    crNor.x = crCen.x - 0.0;
    crNor.y = crCen.y - 0.0;
    crNor.z = 0.0;
    sgSpaceMath::NormalVector(crNor);
    SG_CIRCLE  crGeo;
    crGeo.FromCenterRadiusNormal(crCen, 1, crNor);
    sgCCircle* cr = sgCreateCircle(crGeo);
    sgGetScene()->AttachObject(cr);
    cr->SetAttribute(SG_OA_COLOR,((int)(10*i))%200+50);
    cr->SetAttribute(SG_OA_LINE_THICKNESS, ((int)(10*i))%5);
    cr->SetAttribute(SG_OA_LINE_TYPE, ((int)(10*i))%5);
  }
}

void OnBnClickedPrArcs()
{
  sgGetScene()->Clear();

  for (double i=0.0;i<2.0*3.14159265;i+=0.8)
  {
    SG_POINT   arP1 = {5.0*cos(i),5.0*sin(i),0.0};
    SG_POINT   arP2 = {5.0*cos(i+0.4),5.0*sin(i+0.4),0.0};
    SG_POINT   arP3 = {7.0*cos(i+0.2),7.0*sin(i+0.2),3.0};
    SG_ARC     arGeo;
    if (arGeo.FromThreePoints(arP1, arP2, arP3,false))
    {
      sgCArc* ar = sgCreateArc(arGeo);
      if (ar)
      {
        sgGetScene()->AttachObject(ar);
        ar->SetAttribute(SG_OA_COLOR,((int)(10*i))%200+50);
        ar->SetAttribute(SG_OA_LINE_THICKNESS, ((int)(10*i))%5);
        ar->SetAttribute(SG_OA_LINE_TYPE, ((int)(10*i))%5);
      }
    }
  }
}

void OnBnClickedPrSplines()
{
  sgGetScene()->Clear();

  SG_SPLINE* spl1 = SG_SPLINE::Create();
  SG_POINT tmpPnt = {0.0, 0.0, 0.0};

  int fl=0;
  for (double i=0.0;i<2.0*3.14159265;i+=0.4)
  {
    tmpPnt.x = ((double)(fl%3))*cos(i);
    tmpPnt.y = ((double)(fl%3))*sin(i);
    tmpPnt.z = (fl%2)?1.0:-1.0;
    spl1->AddKnot(tmpPnt,fl);
    fl++;
  }

  sgCSpline* spl1_obj = sgCreateSpline(*spl1);
  sgGetScene()->AttachObject(spl1_obj);
  spl1_obj->SetAttribute(SG_OA_COLOR,10);
  spl1_obj->SetAttribute(SG_OA_LINE_THICKNESS, 2);

  SG_SPLINE::Delete(spl1);

  SG_SPLINE* spl2 = SG_SPLINE::Create();
  fl=0;
  for (double i=0.0;i<2.0*3.14159265;i+=0.4)
  {
    tmpPnt.x = ((double)(fl%3+2))*cos(i);
    tmpPnt.y = ((double)(fl%3+2))*sin(i);
    tmpPnt.z = 0.0;
    spl2->AddKnot(tmpPnt,fl);
    fl++;
  }
  spl2->Close();

  sgCSpline* spl2_obj = sgCreateSpline(*spl2);
  sgGetScene()->AttachObject(spl2_obj);
  spl2_obj->SetAttribute(SG_OA_COLOR,0);
  spl2_obj->SetAttribute(SG_OA_LINE_THICKNESS, 1);

  SG_SPLINE::Delete(spl2);
}

void OnBnClickedPrContours()
{
  sgGetScene()->Clear();

  std::vector<sgCObject*> objts;

  for (double i=0.0;i<2.0*3.14159265;i+=2.0*3.14159265/12.0)
  {
    SG_POINT   arP1 = {5.0*cos(i),5.0*sin(i),0.0};
    SG_POINT   arP2 = {5.0*cos(i+2.0*3.14159265/24.0),5.0*sin(i+2.0*3.14159265/24.0),0.0};
    SG_POINT   arP3 = {7.0*cos(i+2.0*3.14159265/48.0),7.0*sin(i+2.0*3.14159265/48.0),0.0};
    SG_ARC     arGeo;
    if (arGeo.FromThreePoints(arP1, arP2, arP3,false))
    {
      sgCArc* ar = sgCreateArc(arGeo);
      if (ar)
      {
        objts.push_back(ar);
      }
    }
    objts.push_back(sgCreateLine(5.0*cos(i+2.0*3.14159265/24.0),5.0*sin(i+2.0*3.14159265/24.0),0.0,
      5.0*cos(i+2.0*3.14159265/12.0),5.0*sin(i+2.0*3.14159265/12.0),0.0));
  }

  sgCContour* cnt1 = sgCContour::CreateContour(&objts[0],objts.size());
  objts.clear();
  sgGetScene()->AttachObject(cnt1);
  cnt1->SetAttribute(SG_OA_COLOR,0);
  cnt1->SetAttribute(SG_OA_LINE_THICKNESS, 1);


  for (double i=0.0;i<2.0*3.14159265;i+=2.0*3.14159265/6.0)
  {
    SG_POINT   arP1 = {2.0*cos(i),2.0*sin(i),0.0};
    SG_POINT   arP2 = {2.0*cos(i+2.0*3.14159265/12.0),2.0*sin(i+2.0*3.14159265/12.0),0.0};
    SG_POINT   arP3 = {3.0*cos(i+2.0*3.14159265/24.0),3.0*sin(i+2.0*3.14159265/24.0),2.0};
    SG_ARC     arGeo;
    if (arGeo.FromThreePoints(arP1, arP2, arP3,false))
    {
      sgCArc* ar = sgCreateArc(arGeo);
      if (ar)
      {
        objts.push_back(ar);
      }
    }
    objts.push_back(sgCreateLine(2.0*cos(i+2.0*3.14159265/12.0),2.0*sin(i+2.0*3.14159265/12.0),0.0,
      2.0*cos(i+2.0*3.14159265/6.0),2.0*sin(i+2.0*3.14159265/6.0),0.0));
  }

  sgCContour* cnt2 = sgCContour::CreateContour(&objts[0],objts.size());
  objts.clear();
  sgGetScene()->AttachObject(cnt2);
  cnt2->SetAttribute(SG_OA_COLOR,10);
  cnt2->SetAttribute(SG_OA_LINE_THICKNESS, 2);
}

void OnBnClickedPrEquidi()
{
  sgCObject* curObj = sgGetScene()->GetObjectsList()->GetHead();
  std::vector<sgC2DObject*>   good_objcts;
  int i=0;
  while (curObj)
  {
    SG_OBJECT_TYPE ot = curObj->GetType();
    if (ot==SG_OT_CIRCLE ||ot==SG_OT_ARC || ot==SG_OT_SPLINE || ot==SG_OT_CONTOUR)
    {
      sgC2DObject* o2D = reinterpret_cast<sgC2DObject*>(curObj);
      if (!o2D->IsLinear() && o2D->IsPlane(NULL,NULL))
        good_objcts.push_back(o2D);
    }
    curObj = sgGetScene()->GetObjectsList()->GetNext(curObj);
 }

  size_t sz = good_objcts.size();
  for (size_t i=0;i<sz;i++)
  {
    sgCContour* eq1 = good_objcts[i]->GetEquidistantContour(0.3, 0.3, false);
    if (eq1)
      sgGetScene()->AttachObject(eq1);
    sgCContour* eq2 = good_objcts[i]->GetEquidistantContour(-0.3, -0.3, false);
    if (eq2)
      sgGetScene()->AttachObject(eq2);
  }
}

void OnBnClickedPrBoxes()
{
  sgGetScene()->Clear();

  AddFloorInScene(7.0, 7.0, 1.0, 1.0, -0.3);

  sgCBox* bx1 = sgCreateBox(1,2,2.1);

  SG_VECTOR transV = {-5,-5,0};
 /* bx1->InitTempMatrix()->Translate(transV);
  bx1->ApplyTempMatrix();
  bx1->DestroyTempMatrix();*/

  sgGetScene()->AttachObject(bx1);
  bx1->SetAttribute(SG_OA_COLOR,8);

  sgCBox* bx2 = sgCreateBox(1,1,1);

  transV.x = 1.5;
  transV.y = 0.0;
  transV.z = 0.0;
  bx2->InitTempMatrix()->Translate(transV);
  bx2->ApplyTempMatrix();
  bx2->DestroyTempMatrix();

  sgGetScene()->AttachObject(bx2);
  bx2->SetAttribute(SG_OA_COLOR,0);
  bx2->SetAttribute(SG_OA_LINE_THICKNESS,1);
  bx2->SetAttribute(SG_OA_LINE_TYPE,1);
}

void OnBnClickedPrSpheres()
{
  sgGetScene()->Clear();

  AddFloorInScene(20.0, 10.0, 0.0, 1.0, -2.0);

  for (int i=3;i<5;i++)
  {
      sgCSphere* sp1 = sgCreateSphere(i,24,24);

      SG_VECTOR transV1 = {5*(i-3),2,0};
      sp1->InitTempMatrix()->Translate(transV1);
      sp1->ApplyTempMatrix();
      sp1->DestroyTempMatrix();

      sgGetScene()->AttachObject(sp1);
      sp1->SetAttribute(SG_OA_COLOR,i);
      if (i%2)
        sp1->SetAttribute(SG_OA_DRAW_STATE,SG_DS_FRAME);
  }
}

void OnBnClickedPrCyls()
{
  sgGetScene()->Clear();

  AddFloorInScene(80.0, 50.0, 20.0, 10.0, 5.0);

  for (int i=2;i<8;i++)
  {
    sgCCylinder* cy1 = sgCreateCylinder(i,2*i,24);
    SG_VECTOR transV1 = {5*i,10,10};
    cy1->InitTempMatrix()->Translate(transV1);
    cy1->ApplyTempMatrix();
    cy1->DestroyTempMatrix();

    sgGetScene()->AttachObject(cy1);
    cy1->SetAttribute(SG_OA_COLOR,i);

    if (i%2)
      cy1->SetAttribute(SG_OA_DRAW_STATE,SG_DS_FRAME);
  }
}

void OnBnClickedPrCones()
{
  sgGetScene()->Clear();

  AddFloorInScene(200.0, 100.0, 50.0, 10.0, 5.0);

  for (int i=2;i<10;i++)
  {
    sgCCone* co1 = sgCreateCone(i,i/3,10*i,36);
    SG_VECTOR transV1 = {10*i,10,10};
    co1->InitTempMatrix()->Translate(transV1);
    co1->ApplyTempMatrix();
    co1->DestroyTempMatrix();

    sgGetScene()->AttachObject(co1);
    co1->SetAttribute(SG_OA_COLOR,i);
  }
}

void OnBnClickedPrTors()
{
  sgGetScene()->Clear();

  AddFloorInScene(100.0, 50.0, 50.0, 10.0, 2.0);

  for (int i=2;i<10;i++)
  {
    sgCTorus* tor1 = sgCreateTorus(i,i/6+2,24,24);
    SG_POINT rotCen = {0.0,0.0,0.0};
    SG_VECTOR rotDir = {rand(),rand(),rand()};
    tor1->InitTempMatrix()->Rotate(rotCen,rotDir, rand()*360);
    SG_VECTOR transV1 = {10*i,10,10};
    tor1->GetTempMatrix()->Translate(transV1);
    tor1->ApplyTempMatrix();
    tor1->DestroyTempMatrix();

    sgGetScene()->AttachObject(tor1);
    tor1->SetAttribute(SG_OA_COLOR,i);
  }

}

void OnBnClickedPrEllipsoids()
{
  sgGetScene()->Clear();

  AddFloorInScene(100.0, 50.0, 50.0, 10.0, 2.0);

  for (int i=3;i<10;i++)
  {
    sgCEllipsoid* ell1 = sgCreateEllipsoid(i,i/6+2,2*i, 24,24);
       SG_POINT rotCen = {0.0,0.0,0.0};
    SG_VECTOR rotDir = {rand(),rand(),rand()};
    ell1->InitTempMatrix()->Rotate(rotCen,rotDir, rand()*360);
    SG_VECTOR transV1 = {10*i,10,10};
    ell1->GetTempMatrix()->Translate(transV1);
    ell1->ApplyTempMatrix();
    ell1->DestroyTempMatrix();

    sgGetScene()->AttachObject(ell1);
    ell1->SetAttribute(SG_OA_COLOR,i);
    
  }
}

void OnBnClickedPrSphericBands()
{
  sgGetScene()->Clear();

  AddFloorInScene(40.0, 40.0, 15.0, 10.0, 0.0);

  for (int i=2;i<5;i++)
  {
    sgCSphericBand* sb1 = sgCreateSphericBand(i,-0.4, 0.5, 24);
    SG_VECTOR transV1 = {5*i,10,10};
    sb1->InitTempMatrix()->Translate(transV1);
    sb1->ApplyTempMatrix();
    sb1->DestroyTempMatrix();

    sgGetScene()->AttachObject(sb1);
    sb1->SetAttribute(SG_OA_COLOR,i);
  }

}



void  AddFloorInScene(double size1, double size2, 
					  double x_shift, double y_shift, double z_shift)
{
	sgCObject*   objects_buffer[4];
	objects_buffer[0] = sgCreateLine(size1/2.0, size2/2.0, 0.0, 
									 -size1/2.0, size2/2.0, 0.0);
	objects_buffer[1] = sgCreateLine(-size1/2.0, size2/2.0, 0.0, 
									 -size1/2.0, -size2/2.0, 0.0);
	objects_buffer[2] = sgCreateLine(-size1/2.0, -size2/2.0, 0.0, 
									 size1/2.0, -size2/2.0, 0.0);
	objects_buffer[3] = sgCreateLine(size1/2.0, -size2/2.0, 0.0, 
									 size1/2.0, size2/2.0, 0.0);
	
	sgCContour*  cnt = sgCContour::CreateContour(objects_buffer,4);
	
	sgCObject*   floor = sgSurfaces::Face((const sgC2DObject&)(*cnt),NULL,0);
	sgDeleteObject(cnt);
	
	sgGetScene()->AttachObject(floor);
	floor->SetAttribute(SG_OA_COLOR,25);
	
	SG_VECTOR transV = {x_shift, y_shift, z_shift};
	floor->InitTempMatrix()->Translate(transV);
	floor->ApplyTempMatrix();
	floor->DestroyTempMatrix();
	
}